// Copyright (c) 2012 Sirikata Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can
// be found in the LICENSE file.


#include <cxxtest/TestSuite.h>

#include <sirikata/core/util/Thread.hpp>

#include <sirikata/core/transfer/URI.hpp>

#include <sirikata/core/transfer/TransferPool.hpp>
#include <sirikata/core/transfer/AggregatedTransferPool.hpp>
#include <sirikata/core/transfer/RemoteFileMetadata.hpp>
#include <sirikata/core/transfer/TransferMediator.hpp>

#include <sirikata/core/transfer/DiskManager.hpp>
#include <sirikata/core/transfer/LRUPolicy.hpp>

#include <sirikata/core/network/Address.hpp>
#include <sirikata/core/transfer/HttpManager.hpp>

#include <sirikata/core/options/CommonOptions.hpp>

// NOTE: This depends on being added after TransferTest.

class UploadVerifier
    : public RequestVerifier {
public:
    typedef std::map<String, String> StringMap;

private:
    Transfer::OAuthParamsPtr mOAuth;
    const StringMap mFiles; // Filename -> data
    const String mPath; // Requested location of asset,
                        // e.g. apiupload/filename results in
                        // /username/apiupload/filename
    const StringMap mParams; // Optional params, e.g. title = 'Building'

    const bool mExpectedSuccess;
    const String mExpectedFullPath;
public:
    UploadVerifier(
        Transfer::OAuthParamsPtr oauth, const StringMap& files,
        const String& path, const StringMap& params,
        bool success_expected, const String& expected_full_path)
     : mOAuth(oauth), mFiles(files), mPath(path), mParams(params),
       mExpectedSuccess(success_expected), mExpectedFullPath(expected_full_path)
    {
    }

    virtual ~UploadVerifier() {}

    void addToPool(Transfer::TransferPoolPtr pool,
        VerifyFinished cb, Transfer::Priority priority) {
        Transfer::TransferRequestPtr req(
            new Transfer::UploadRequest(
                mOAuth,
                mFiles, mPath, mParams, 1.0f,
                std::tr1::bind(
                    &UploadVerifier::uploadFinished, this, std::tr1::placeholders::_1, std::tr1::placeholders::_2, cb
                )
            )
        );
        pool->addRequest(req);
    }

    void uploadFinished(
        Transfer::UploadRequestPtr request,
        const Transfer::URI& path,
        VerifyFinished cb)
    {
        bool succeeded = !path.empty();
        TS_ASSERT_EQUALS(succeeded, mExpectedSuccess);

        if (succeeded) {
            // Make sure we got the path we were expecting
            String path_str = path.toString();
            TS_ASSERT(path_str.find(mPath) != String::npos);
            // We don't assert equality here because the URL generated
            // has a version number we can't actually
            // predict. However, it should start with the value passed
            // in.
            TS_ASSERT(path_str.find(mExpectedFullPath) == 0);
        }

        // This decrements the outstanding request count, we need to
        // do it regardless of the result
        cb();
    }
};


class UploadTest : public CxxTest::TestSuite {
    //Mediates transfers between subsystems (graphics, physics, etc)
    Transfer::TransferMediator* mTransferMediator;
    Transfer::OAuthParamsPtr mOauth;

    SampleClient* mSampleClient1;

    Thread* mClientThread1;

public:

    UploadTest()
    {
        InitOptions(); // For CDN settings
        FakeParseOptions();
        mTransferMediator = &Transfer::TransferMediator::getSingleton();
    }

    void setUp() {
        String oauth_hostname = "localhost";
        String oauth_service = "8000";
        String oauth_consumer_key = "j7oc4pLytjsDewPFI3ulKjtvbhOvX1iLGgDQ06_Z774=";
        String oauth_consumer_secret = "ryQYPxRB2uxvM5PA02eJzRPT5rEhFvzeYfZwjWykir4=";
        String oauth_access_key = "lTIQZ9YWOp8HGhyyWmqrD_rCFb-EZlL4LPgFRZuK5ws=";
        String oauth_access_secret = "ymY2SC3HjzFnhL_oWf6XX7h-9YNTpPn0Bn08kZ697ro=";

        String oauth_server_prefix = "meerkat://" + oauth_hostname + ":" + oauth_service;

        mOauth = Transfer::OAuthParamsPtr(
            new Transfer::OAuthParams(
                oauth_hostname, oauth_service,
                oauth_consumer_key, oauth_consumer_secret,
                oauth_access_key, oauth_access_secret
            )
        );

        std::vector<RequestVerifierPtr> list1;
        // Valid zip upload, with no optional params to test that
        // missing params work
        {
            Transfer::UploadRequest::StringMap files;
            files["multimtl.zip"] = String((const char*)testColladaZip, testColladaZipLength);

            Transfer::UploadRequest::StringMap params;
            params["username"] = "echeslack";
            params["title"] = "Multimaterial";
            params["main_filename"] = "multimtl.zip";
            // Path is specified in the upload call

            list1.push_back(
                RequestVerifierPtr(
                    new UploadVerifier(
                        mOauth,
                        files,
                        "apiupload/multimtl.dae",
                        params,
                        true, oauth_server_prefix + "/echeslack/apiupload/multimtl.dae"
                    )
                )
            );
        }

        // Valid multi-file upload
        {
            Transfer::UploadRequest::StringMap files;
            files["dice.dae"] = String((const char*)diceDae, diceDaeLength);
            files["dice.png"] = String((const char*)diceTexture, diceTextureLength);

            Transfer::UploadRequest::StringMap params;
            params["username"] = "echeslack";
            params["title"] = "Dice";
            params["labels"] = "test, collada, dice, simple";
            params["description"] = "Simple textured dice model";
            params["main_filename"] = "dice.dae";
            // Path is specified in the upload call

            list1.push_back(
                RequestVerifierPtr(
                    new UploadVerifier(
                        mOauth,
                        files,
                        "apiupload/dice.dae",
                        params,
                        true, oauth_server_prefix + "/echeslack/apiupload/dice.dae"
                    )
                )
            );
        }

        // Invalid upload -- no main_filename to trigger negative response
        {
            Transfer::UploadRequest::StringMap files;
            files["dice.dae"] = String((const char*)diceDae, diceDaeLength);
            files["dice.png"] = String((const char*)diceTexture, diceTextureLength);

            Transfer::UploadRequest::StringMap params;
            params["username"] = "echeslack";
            params["title"] = "Dice";
            params["labels"] = "test, collada, dice, simple";
            params["description"] = "Simple textured dice model";
            // Removed to trigger failure: params["main_filename"] = "dice.dae";
            // Path is specified in the upload call

            list1.push_back(
                RequestVerifierPtr(
                    new UploadVerifier(
                        mOauth,
                        files,
                        "apiupload/dice_failure_no_main_filename.dae",
                        params,
                        false, ""
                    )
                )
            );
        }

        // Invalid upload -- no files to trigger failure
        {
            Transfer::UploadRequest::StringMap files;

            Transfer::UploadRequest::StringMap params;
            params["username"] = "echeslack";
            params["title"] = "Dice";
            params["labels"] = "test, collada, dice, simple";
            params["description"] = "Simple textured dice model";
            params["main_filename"] = "dice.dae";
            // Path is specified in the upload call

            list1.push_back(
                RequestVerifierPtr(
                    new UploadVerifier(
                        mOauth,
                        files,
                        "apiupload/dice_failure_no_files.dae",
                        params,
                        false, ""
                    )
                )
            );
        }

        mSampleClient1 = new SampleClient(*mTransferMediator, "sample1", list1);
        mClientThread1 = new Thread(std::tr1::bind(&SampleClient::run, mSampleClient1));
    }

    void tearDown() {
        //Make sure clients have exited
        mClientThread1->join();

        //Wait for transfer mediator thread to exit
        mTransferMediator->cleanup();
    }

    void testTransferRequests() {
        srand ( time(NULL) );
        boost::unique_lock<boost::mutex> lock(mut);
        done.wait(lock);
    }

private:

    // This is the multimtl sample, as a zip file, converted into C++
    // so we can use it easily in this test.
    static const unsigned char testColladaZip[];
    static const int32 testColladaZipLength = 2361;

    // Dice model, as separate files so that we can test multiple file
    // uploads
    static const unsigned char diceDae[];
    static const int32 diceDaeLength = 5174;

    static const unsigned char diceTexture[];
    static const int32 diceTextureLength = 15847;
};

const unsigned char UploadTest::testColladaZip[] = {
        80,75,3,4,20,0,0,0,8,0,19,132,178,62,160,82,15,162,191,8,0,0,221,36,0
        ,0,12,0,0,0,109,117,108,116,105,109,116,108,46,100,97,101,237,26,107,111,219,56,242,187
        ,127,5,161,251,114,7,212,214,195,143,216,169,98,192,77,211,189,236,37,113,17,167,221,164,95
        ,12,70,166,109,162,18,233,146,148,19,31,238,199,223,144,122,219,146,147,238,245,112,183,216,2
        ,81,194,25,14,135,51,195,121,137,138,127,62,189,186,154,188,159,160,231,40,100,242,204,90,43
        ,181,57,181,237,167,167,167,78,192,195,16,47,112,135,139,149,237,57,78,223,118,93,59,165,158
        ,5,107,18,97,11,109,137,144,148,179,51,203,237,244,58,174,53,110,33,228,99,41,137,210,35
        ,24,7,156,41,65,31,99,197,69,130,209,243,177,90,3,136,67,46,40,227,190,157,194,213,105
        ,202,86,115,197,121,56,190,198,59,140,78,58,14,250,23,58,79,228,49,152,173,215,113,92,244
        ,107,204,16,26,33,16,110,128,176,66,238,224,212,25,158,186,35,160,253,144,18,163,173,219,113
        ,221,108,147,156,107,182,89,192,163,136,48,37,199,25,185,97,126,241,188,225,66,161,233,70,129
        ,110,242,20,61,226,175,228,78,96,38,151,92,68,242,204,121,75,12,193,71,30,238,86,156,93
        ,19,185,38,242,204,125,171,201,174,232,106,173,96,27,32,162,114,134,163,77,152,0,173,32,22
        ,91,114,14,236,148,192,148,149,102,18,94,231,56,34,2,79,228,21,231,95,177,210,11,18,188
        ,225,167,153,151,201,10,248,87,78,65,250,9,91,204,190,82,6,216,116,213,132,209,8,27,225
        ,115,202,59,65,49,91,133,164,16,255,146,109,169,164,143,33,185,225,11,131,78,23,223,128,146
        ,56,44,173,36,207,231,156,139,69,129,249,76,132,210,72,56,193,18,25,176,215,166,44,24,193
        ,194,18,50,85,32,179,64,9,247,113,189,147,52,40,224,251,91,178,212,124,91,11,34,200,18
        ,30,22,144,4,231,188,13,140,1,238,63,240,109,14,60,104,0,142,56,63,203,226,112,55,59
        ,161,237,55,110,157,103,195,196,87,102,156,237,192,157,162,77,172,136,64,23,12,126,43,144,73
        ,47,71,151,44,232,180,174,104,64,152,36,11,20,51,16,2,169,53,65,179,243,139,9,154,173
        ,177,0,236,140,199,34,32,40,165,122,131,62,39,97,128,92,240,211,191,2,113,203,74,167,172
        ,191,189,69,59,30,163,8,239,16,227,10,197,146,0,51,42,209,146,134,4,145,231,128,108,20
        ,162,12,129,236,224,15,24,20,69,79,84,173,245,134,153,12,29,244,144,50,224,143,90,72,132
        ,145,86,12,241,165,17,43,165,2,247,63,109,165,177,43,136,36,88,4,235,142,12,8,134,40
        ,142,108,61,152,75,35,251,92,26,217,231,97,202,125,173,162,16,181,62,49,240,12,137,4,249
        ,22,83,173,224,227,14,225,13,72,20,96,240,15,20,226,39,196,5,194,43,65,96,78,113,45
        ,241,147,160,218,207,223,32,201,151,234,9,56,183,22,84,38,177,94,49,91,38,31,232,92,38
        ,0,99,97,134,172,201,12,93,206,44,244,110,50,187,156,189,105,253,118,121,247,247,233,167,59
        ,244,219,228,246,118,114,115,119,121,49,67,211,91,116,62,189,121,127,121,119,57,189,1,232,3
        ,154,220,60,160,127,92,222,188,127,131,8,24,10,54,1,159,17,90,118,16,144,106,35,146,69
        ,167,53,35,164,178,57,196,173,129,229,134,4,116,73,3,208,136,173,98,188,34,104,197,33,133
        ,49,80,4,109,136,136,168,212,199,40,65,180,69,43,164,17,85,73,16,29,170,211,105,105,127
        ,203,221,203,184,155,125,144,236,252,64,16,12,218,142,181,211,181,225,199,115,239,60,247,212,235
        ,159,118,79,190,0,125,58,155,208,70,124,1,162,53,18,231,211,9,117,204,168,66,17,1,199
        ,61,179,28,72,133,22,98,16,12,103,22,72,167,168,193,91,118,70,186,153,227,103,42,199,15
        ,243,79,31,125,59,131,116,166,182,243,84,237,135,244,81,96,177,155,39,33,37,51,249,13,132
        ,232,2,24,135,243,152,233,61,22,243,98,179,50,46,15,59,14,105,51,200,163,16,16,138,4
        ,107,70,191,197,100,174,67,148,179,98,10,38,193,236,250,84,20,221,146,50,30,102,118,75,190
        ,29,119,79,58,195,222,168,235,219,6,170,18,96,179,112,46,244,33,141,117,142,47,195,85,210
        ,127,50,136,136,177,11,150,242,237,100,188,55,191,52,211,78,66,176,172,206,251,118,173,144,190
        ,221,164,152,111,151,109,0,7,109,204,120,104,83,69,100,154,209,33,175,108,72,102,214,125,244
        ,127,207,178,125,199,253,49,150,213,46,248,191,181,172,111,215,184,112,238,214,161,41,163,233,42
        ,3,152,3,184,82,115,83,96,219,64,151,89,191,130,203,247,109,182,180,191,209,53,184,162,85
        ,160,43,227,88,23,131,244,209,185,33,228,98,143,8,42,33,102,106,142,149,34,44,54,153,102
        ,156,210,214,204,148,151,66,231,0,134,174,76,27,243,214,224,203,203,190,197,120,161,143,48,56
        ,92,89,63,85,58,144,138,146,77,231,163,37,40,242,97,97,102,179,216,24,213,56,180,91,54
        ,119,237,220,79,179,255,78,179,23,81,80,120,124,30,4,208,18,18,232,2,195,44,14,50,216
        ,156,209,187,48,206,19,144,25,231,187,82,99,22,104,25,200,114,9,145,138,98,17,158,89,127
        ,209,52,237,229,115,94,101,236,140,91,29,243,91,178,200,120,235,225,11,172,129,164,145,115,161
        ,96,69,155,92,199,132,81,166,97,202,54,211,79,115,45,169,168,193,92,148,141,224,186,39,155
        ,159,79,175,175,167,55,117,57,22,73,83,7,141,225,173,202,9,111,214,156,173,246,242,30,73
        ,155,137,42,58,247,82,56,126,148,61,245,158,170,117,173,103,226,227,232,145,146,170,243,127,55
        ,235,90,30,62,244,24,75,232,81,27,57,187,221,19,175,223,7,214,61,167,59,242,60,24,12
        ,79,156,254,112,120,100,163,90,150,190,78,250,113,184,95,45,74,59,233,77,146,167,153,117,61
        ,19,95,174,41,116,116,208,19,30,48,95,134,28,171,180,13,72,198,251,28,235,151,250,240,30
        ,18,214,149,210,239,52,122,19,155,130,63,85,187,6,169,193,20,13,66,55,47,246,149,126,107
        ,221,64,119,254,31,122,75,35,159,210,14,65,147,228,110,163,185,155,22,67,226,171,134,84,41
        ,245,21,217,176,46,100,33,102,76,204,31,38,128,52,171,20,105,232,103,248,127,119,248,15,32
        ,212,189,30,176,118,29,119,52,112,205,96,52,116,122,63,62,252,187,131,225,192,59,65,135,131
        ,159,217,224,103,54,248,93,217,160,232,28,74,61,66,222,55,172,8,135,183,102,65,73,214,58
        ,164,136,157,201,29,143,252,185,220,179,2,88,100,142,136,200,117,73,180,228,122,165,188,170,189
        ,225,146,154,123,132,162,231,77,16,213,52,98,108,50,199,66,224,93,253,242,182,153,179,80,192
        ,99,166,206,44,175,103,141,219,125,7,237,253,180,211,167,128,14,254,148,168,218,213,73,120,210
        ,195,73,4,169,8,120,236,109,83,167,165,32,128,80,225,34,147,111,104,161,196,24,208,210,53
        ,234,162,111,132,22,64,209,181,14,220,5,60,1,71,72,237,54,48,109,68,202,204,119,159,181
        ,134,175,33,126,248,30,226,47,251,196,144,40,83,173,170,14,121,228,189,196,78,148,62,238,18
        ,44,185,99,205,246,77,192,215,185,67,186,116,207,25,78,60,107,12,33,139,32,116,107,126,183
        ,27,255,52,207,52,115,59,206,177,118,221,15,115,42,112,250,67,175,218,51,201,159,193,167,182
        ,68,192,107,35,145,21,215,200,144,85,71,162,108,19,43,36,73,4,239,182,52,56,179,62,78
        ,103,230,58,245,88,120,150,101,246,237,140,111,185,45,202,62,39,100,7,227,58,22,202,94,201
        ,146,215,171,217,47,181,114,52,28,158,85,146,240,102,122,123,61,185,178,16,95,46,37,209,188
        ,171,38,108,226,84,202,179,5,175,207,23,183,119,23,247,5,47,103,143,215,102,156,184,169,139
        ,250,200,51,206,170,255,246,80,23,13,224,247,9,64,122,52,72,49,30,96,28,52,4,104,4
        ,144,235,24,192,12,60,228,186,64,226,106,10,183,11,203,220,94,2,154,17,240,239,235,209,64
        ,207,158,232,21,195,4,52,35,152,31,65,1,171,20,184,204,190,199,108,238,149,77,14,29,237
        ,31,199,226,158,54,152,167,13,166,95,34,53,104,70,46,242,186,47,25,194,183,139,138,235,219
        ,89,153,174,86,248,106,53,207,139,252,150,202,24,135,115,25,16,150,215,249,50,206,4,211,103
        ,131,152,105,88,127,24,203,194,24,12,78,85,88,190,188,96,48,107,86,36,119,181,25,97,10
        ,85,206,13,250,154,16,14,42,121,133,200,65,40,223,110,207,237,12,6,3,52,242,58,189,81
        ,127,136,188,209,160,211,77,59,33,77,82,226,34,184,202,89,36,227,135,44,229,183,189,1,244
        ,125,6,119,116,193,189,53,206,18,117,191,227,142,250,189,87,173,250,98,165,33,226,212,144,231
        ,55,56,233,181,118,114,131,83,249,48,96,23,39,167,77,118,104,63,115,251,151,95,191,26,224
        ,117,214,235,122,16,171,240,244,28,231,181,54,59,170,204,17,19,191,142,60,55,240,81,83,37
        ,23,163,137,165,42,183,205,47,90,234,29,47,46,176,74,61,232,255,173,166,121,19,157,40,155
        ,181,209,149,116,240,72,217,98,94,189,60,44,206,254,104,115,80,222,41,191,110,148,187,232,145
        ,23,21,8,41,44,86,58,11,153,11,203,154,186,221,204,33,73,168,5,3,125,115,121,80,202
        ,143,126,129,177,27,116,243,237,3,11,189,116,242,197,55,161,195,175,68,127,238,92,179,255,181
        ,236,197,32,42,190,56,184,135,31,33,220,215,25,179,171,107,246,31,54,233,212,126,115,169,181
        ,27,244,126,165,250,88,45,178,7,213,212,207,137,202,27,87,10,108,178,255,126,137,181,19,198
        ,197,30,249,167,60,253,69,157,135,33,17,210,174,76,208,8,175,200,30,78,75,173,81,126,246
        ,79,74,227,214,191,1,80,75,1,2,20,3,20,0,0,0,8,0,19,132,178,62,160,82,15
        ,162,191,8,0,0,221,36,0,0,12,0,0,0,0,0,0,0,0,0,0,0,128,1,0,0
        ,0,0,109,117,108,116,105,109,116,108,46,100,97,101,80,75,5,6,0,0,0,0,1,0,1
        ,0,58,0,0,0,233,8,0,0,0,0
    };


const unsigned char UploadTest::diceDae[] = {
60,63,120,109,108,32,118,101,114,115,105,111,110,61,34,49,46,48,34,32,101,110,99,111,100
,105,110,103,61,34,117,116,102,45,56,34,63,62,10,60,67,79,76,76,65,68,65,32,120,109
,108,110,115,61,34,104,116,116,112,58,47,47,119,119,119,46,99,111,108,108,97,100,97,46,111
,114,103,47,50,48,48,53,47,49,49,47,67,79,76,76,65,68,65,83,99,104,101,109,97,34
,32,118,101,114,115,105,111,110,61,34,49,46,52,46,49,34,62,10,32,32,60,97,115,115,101
,116,62,10,32,32,32,32,60,99,111,110,116,114,105,98,117,116,111,114,62,10,32,32,32,32
,32,32,60,97,117,116,104,111,114,62,104,108,97,119,60,47,97,117,116,104,111,114,62,10,32
,32,32,32,32,32,60,97,117,116,104,111,114,105,110,103,95,116,111,111,108,62,77,97,121,97
,50,48,48,56,32,124,32,67,111,108,108,97,100,97,77,97,121,97,32,118,51,46,48,53,66
,60,47,97,117,116,104,111,114,105,110,103,95,116,111,111,108,62,10,32,32,32,32,32,32,60
,99,111,112,121,114,105,103,104,116,62,10,32,32,32,32,32,32,32,32,67,111,112,121,114,105
,103,104,116,32,50,48,48,56,32,83,111,110,121,32,67,111,109,112,117,116,101,114,32,69,110
,116,101,114,116,97,105,110,109,101,110,116,32,73,110,99,46,10,32,32,32,32,32,32,32,32
,76,105,99,101,110,115,101,100,32,117,110,100,101,114,32,116,104,101,32,67,114,101,97,116,105
,118,101,32,67,111,109,109,111,110,115,32,65,116,116,114,105,98,117,116,105,111,110,32,78,111
,110,99,111,109,109,101,114,99,105,97,108,32,83,104,97,114,101,32,65,108,105,107,101,32,108
,105,99,101,110,115,101,46,10,32,32,32,32,32,32,32,32,83,101,101,32,108,105,99,101,110
,115,101,32,102,105,108,101,32,111,114,32,119,119,119,46,99,114,101,97,116,105,118,101,99,111
,109,109,111,110,115,46,111,114,103,32,102,111,114,32,100,101,116,97,105,108,115,46,10,32,32
,32,32,32,32,60,47,99,111,112,121,114,105,103,104,116,62,10,32,32,32,32,60,47,99,111
,110,116,114,105,98,117,116,111,114,62,10,32,32,32,32,60,99,114,101,97,116,101,100,62,50
,48,48,56,45,49,50,45,48,57,84,50,48,58,53,56,58,53,51,90,60,47,99,114,101,97
,116,101,100,62,10,32,32,32,32,60,109,111,100,105,102,105,101,100,62,50,48,48,56,45,49
,50,45,48,57,84,50,49,58,48,55,58,52,54,90,60,47,109,111,100,105,102,105,101,100,62
,10,32,32,32,32,60,117,110,105,116,32,109,101,116,101,114,61,34,48,46,48,49,34,32,110
,97,109,101,61,34,99,101,110,116,105,109,101,116,101,114,34,47,62,10,32,32,32,32,60,117
,112,95,97,120,105,115,62,89,95,85,80,60,47,117,112,95,97,120,105,115,62,10,32,32,60
,47,97,115,115,101,116,62,10,32,32,60,108,105,98,114,97,114,121,95,105,109,97,103,101,115
,62,10,32,32,32,32,60,105,109,97,103,101,32,105,100,61,34,102,105,108,101,49,45,105,109
,97,103,101,34,32,110,97,109,101,61,34,102,105,108,101,49,34,62,10,32,32,32,32,32,32
,60,105,110,105,116,95,102,114,111,109,62,100,105,99,101,46,112,110,103,60,47,105,110,105,116
,95,102,114,111,109,62,10,32,32,32,32,60,47,105,109,97,103,101,62,10,32,32,60,47,108
,105,98,114,97,114,121,95,105,109,97,103,101,115,62,32,32,10,32,32,60,108,105,98,114,97
,114,121,95,109,97,116,101,114,105,97,108,115,62,10,32,32,32,32,60,109,97,116,101,114,105
,97,108,32,105,100,61,34,108,97,109,98,101,114,116,49,34,32,110,97,109,101,61,34,108,97
,109,98,101,114,116,49,34,62,10,32,32,32,32,32,32,60,105,110,115,116,97,110,99,101,95
,101,102,102,101,99,116,32,117,114,108,61,34,35,108,97,109,98,101,114,116,49,45,102,120,34
,47,62,10,32,32,32,32,60,47,109,97,116,101,114,105,97,108,62,10,32,32,60,47,108,105
,98,114,97,114,121,95,109,97,116,101,114,105,97,108,115,62,10,32,32,60,108,105,98,114,97
,114,121,95,101,102,102,101,99,116,115,62,10,32,32,32,32,60,101,102,102,101,99,116,32,105
,100,61,34,108,97,109,98,101,114,116,49,45,102,120,34,62,10,32,32,32,32,32,32,60,112
,114,111,102,105,108,101,95,67,79,77,77,79,78,62,10,32,32,32,32,32,32,32,32,60,110
,101,119,112,97,114,97,109,32,115,105,100,61,34,102,105,108,101,49,45,115,117,114,102,97,99
,101,34,62,10,32,32,32,32,32,32,32,32,32,32,60,115,117,114,102,97,99,101,32,116,121
,112,101,61,34,50,68,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,105,110,105
,116,95,102,114,111,109,62,102,105,108,101,49,45,105,109,97,103,101,60,47,105,110,105,116,95
,102,114,111,109,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,102,111,114,109,97,116
,62,65,56,82,56,71,56,66,56,60,47,102,111,114,109,97,116,62,10,32,32,32,32,32,32
,32,32,32,32,60,47,115,117,114,102,97,99,101,62,10,32,32,32,32,32,32,32,32,60,47
,110,101,119,112,97,114,97,109,62,10,32,32,32,32,32,32,32,32,60,110,101,119,112,97,114
,97,109,32,115,105,100,61,34,102,105,108,101,49,45,115,97,109,112,108,101,114,34,62,10,32
,32,32,32,32,32,32,32,32,32,60,115,97,109,112,108,101,114,50,68,62,10,32,32,32,32
,32,32,32,32,32,32,32,32,60,115,111,117,114,99,101,62,102,105,108,101,49,45,115,117,114
,102,97,99,101,60,47,115,111,117,114,99,101,62,10,32,32,32,32,32,32,32,32,32,32,32
,32,60,109,105,110,102,105,108,116,101,114,62,76,73,78,69,65,82,95,77,73,80,77,65,80
,95,76,73,78,69,65,82,60,47,109,105,110,102,105,108,116,101,114,62,10,32,32,32,32,32
,32,32,32,32,32,32,32,60,109,97,103,102,105,108,116,101,114,62,76,73,78,69,65,82,60
,47,109,97,103,102,105,108,116,101,114,62,10,32,32,32,32,32,32,32,32,32,32,60,47,115
,97,109,112,108,101,114,50,68,62,10,32,32,32,32,32,32,32,32,60,47,110,101,119,112,97
,114,97,109,62,10,32,32,32,32,32,32,32,32,60,116,101,99,104,110,105,113,117,101,32,115
,105,100,61,34,99,111,109,109,111,110,34,62,10,32,32,32,32,32,32,32,32,32,32,60,108
,97,109,98,101,114,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,101,109,105,115
,115,105,111,110,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,99,111,108,111
,114,62,48,32,48,32,48,32,49,60,47,99,111,108,111,114,62,10,32,32,32,32,32,32,32
,32,32,32,32,32,60,47,101,109,105,115,115,105,111,110,62,10,32,32,32,32,32,32,32,32
,32,32,32,32,60,97,109,98,105,101,110,116,62,10,32,32,32,32,32,32,32,32,32,32,32
,32,32,32,60,99,111,108,111,114,62,48,32,48,32,48,32,49,60,47,99,111,108,111,114,62
,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47,97,109,98,105,101,110,116,62,10,32
,32,32,32,32,32,32,32,32,32,32,32,60,100,105,102,102,117,115,101,62,10,32,32,32,32
,32,32,32,32,32,32,32,32,32,32,60,116,101,120,116,117,114,101,32,116,101,120,116,117,114
,101,61,34,102,105,108,101,49,45,115,97,109,112,108,101,114,34,32,116,101,120,99,111,111,114
,100,61,34,84,69,88,48,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47
,100,105,102,102,117,115,101,62,10,32,32,32,32,32,32,32,32,32,32,60,47,108,97,109,98
,101,114,116,62,10,32,32,32,32,32,32,32,32,60,47,116,101,99,104,110,105,113,117,101,62
,10,32,32,32,32,32,32,60,47,112,114,111,102,105,108,101,95,67,79,77,77,79,78,62,10
,32,32,32,32,60,47,101,102,102,101,99,116,62,10,32,32,60,47,108,105,98,114,97,114,121
,95,101,102,102,101,99,116,115,62,10,32,32,60,108,105,98,114,97,114,121,95,103,101,111,109
,101,116,114,105,101,115,62,10,32,32,32,32,60,103,101,111,109,101,116,114,121,32,105,100,61
,34,112,67,117,98,101,83,104,97,112,101,49,34,32,110,97,109,101,61,34,112,67,117,98,101
,83,104,97,112,101,49,34,62,10,32,32,32,32,32,32,60,109,101,115,104,62,10,32,32,32
,32,32,32,32,32,60,115,111,117,114,99,101,32,105,100,61,34,112,67,117,98,101,83,104,97
,112,101,49,45,112,111,115,105,116,105,111,110,115,34,32,110,97,109,101,61,34,112,111,115,105
,116,105,111,110,34,62,10,32,32,32,32,32,32,32,32,32,32,60,102,108,111,97,116,95,97
,114,114,97,121,32,105,100,61,34,112,67,117,98,101,83,104,97,112,101,49,45,112,111,115,105
,116,105,111,110,115,45,97,114,114,97,121,34,32,99,111,117,110,116,61,34,50,52,34,62,45
,53,32,45,53,32,53,32,53,32,45,53,32,53,32,45,53,32,53,32,53,32,53,32,53,32
,53,32,45,53,32,53,32,45,53,32,53,32,53,32,45,53,32,45,53,32,45,53,32,45,53
,32,53,32,45,53,32,45,53,60,47,102,108,111,97,116,95,97,114,114,97,121,62,10,32,32
,32,32,32,32,32,32,32,32,60,116,101,99,104,110,105,113,117,101,95,99,111,109,109,111,110
,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,97,99,99,101,115,115,111,114,32,115
,111,117,114,99,101,61,34,35,112,67,117,98,101,83,104,97,112,101,49,45,112,111,115,105,116
,105,111,110,115,45,97,114,114,97,121,34,32,99,111,117,110,116,61,34,56,34,32,115,116,114
,105,100,101,61,34,51,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,112
,97,114,97,109,32,110,97,109,101,61,34,88,34,32,116,121,112,101,61,34,102,108,111,97,116
,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,112,97,114,97,109,32
,110,97,109,101,61,34,89,34,32,116,121,112,101,61,34,102,108,111,97,116,34,47,62,10,32
,32,32,32,32,32,32,32,32,32,32,32,32,32,60,112,97,114,97,109,32,110,97,109,101,61
,34,90,34,32,116,121,112,101,61,34,102,108,111,97,116,34,47,62,10,32,32,32,32,32,32
,32,32,32,32,32,32,60,47,97,99,99,101,115,115,111,114,62,10,32,32,32,32,32,32,32
,32,32,32,60,47,116,101,99,104,110,105,113,117,101,95,99,111,109,109,111,110,62,10,32,32
,32,32,32,32,32,32,60,47,115,111,117,114,99,101,62,10,32,32,32,32,32,32,32,32,60
,115,111,117,114,99,101,32,105,100,61,34,112,67,117,98,101,83,104,97,112,101,49,45,110,111
,114,109,97,108,115,34,32,110,97,109,101,61,34,110,111,114,109,97,108,34,62,10,32,32,32
,32,32,32,32,32,32,32,60,102,108,111,97,116,95,97,114,114,97,121,32,105,100,61,34,112
,67,117,98,101,83,104,97,112,101,49,45,110,111,114,109,97,108,115,45,97,114,114,97,121,34
,32,99,111,117,110,116,61,34,55,50,34,62,48,32,48,32,49,32,48,32,48,32,49,32,48
,32,48,32,49,32,48,32,48,32,49,32,48,32,49,32,48,32,48,32,49,32,48,32,48,32
,49,32,48,32,48,32,49,32,48,32,48,32,48,32,45,49,32,48,32,48,32,45,49,32,48
,32,48,32,45,49,32,48,32,48,32,45,49,32,48,32,45,49,32,48,32,48,32,45,49,32
,48,32,48,32,45,49,32,48,32,48,32,45,49,32,48,32,49,32,48,32,48,32,49,32,48
,32,48,32,49,32,48,32,48,32,49,32,48,32,48,32,45,49,32,48,32,48,32,45,49,32
,48,32,48,32,45,49,32,48,32,48,32,45,49,32,48,32,48,60,47,102,108,111,97,116,95
,97,114,114,97,121,62,10,32,32,32,32,32,32,32,32,32,32,60,116,101,99,104,110,105,113
,117,101,95,99,111,109,109,111,110,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,97
,99,99,101,115,115,111,114,32,115,111,117,114,99,101,61,34,35,112,67,117,98,101,83,104,97
,112,101,49,45,110,111,114,109,97,108,115,45,97,114,114,97,121,34,32,99,111,117,110,116,61
,34,50,52,34,32,115,116,114,105,100,101,61,34,51,34,62,10,32,32,32,32,32,32,32,32
,32,32,32,32,32,32,60,112,97,114,97,109,32,110,97,109,101,61,34,88,34,32,116,121,112
,101,61,34,102,108,111,97,116,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32
,32,60,112,97,114,97,109,32,110,97,109,101,61,34,89,34,32,116,121,112,101,61,34,102,108
,111,97,116,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,112,97,114
,97,109,32,110,97,109,101,61,34,90,34,32,116,121,112,101,61,34,102,108,111,97,116,34,47
,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47,97,99,99,101,115,115,111,114,62
,10,32,32,32,32,32,32,32,32,32,32,60,47,116,101,99,104,110,105,113,117,101,95,99,111
,109,109,111,110,62,10,32,32,32,32,32,32,32,32,60,47,115,111,117,114,99,101,62,10,32
,32,32,32,32,32,32,32,60,115,111,117,114,99,101,32,105,100,61,34,112,67,117,98,101,83
,104,97,112,101,49,45,109,97,112,49,34,32,110,97,109,101,61,34,109,97,112,49,34,62,10
,32,32,32,32,32,32,32,32,32,32,60,102,108,111,97,116,95,97,114,114,97,121,32,105,100
,61,34,112,67,117,98,101,83,104,97,112,101,49,45,109,97,112,49,45,97,114,114,97,121,34
,32,99,111,117,110,116,61,34,52,56,34,62,48,46,51,51,51,51,51,51,32,48,32,48,46
,54,54,54,54,54,54,32,48,32,48,46,51,51,51,51,51,51,32,49,32,48,46,54,54,54
,54,54,54,32,48,46,53,32,48,46,51,51,51,51,51,51,32,48,46,53,32,48,32,48,46
,53,32,48,46,54,54,54,54,54,54,32,48,46,53,32,48,46,54,54,54,54,54,54,32,49
,32,48,46,54,54,54,54,54,54,32,49,32,49,32,49,32,49,32,48,32,49,46,48,48,48
,50,49,32,48,46,53,32,48,32,48,32,48,32,48,46,53,32,49,32,48,46,53,32,48,46
,54,54,54,54,54,54,32,48,32,48,46,51,51,51,51,51,51,32,49,32,48,46,51,51,51
,51,51,51,32,48,32,48,46,54,54,54,54,54,54,32,48,46,53,32,48,46,51,51,51,51
,51,51,32,48,46,53,32,48,46,54,54,54,54,54,54,32,48,46,53,32,48,32,49,32,48
,46,51,51,51,51,51,51,32,48,46,53,32,48,46,51,51,51,51,51,51,32,48,46,53,60
,47,102,108,111,97,116,95,97,114,114,97,121,62,10,32,32,32,32,32,32,32,32,32,32,60
,116,101,99,104,110,105,113,117,101,95,99,111,109,109,111,110,62,10,32,32,32,32,32,32,32
,32,32,32,32,32,60,97,99,99,101,115,115,111,114,32,115,111,117,114,99,101,61,34,35,112
,67,117,98,101,83,104,97,112,101,49,45,109,97,112,49,45,97,114,114,97,121,34,32,99,111
,117,110,116,61,34,50,52,34,32,115,116,114,105,100,101,61,34,50,34,62,10,32,32,32,32
,32,32,32,32,32,32,32,32,32,32,60,112,97,114,97,109,32,110,97,109,101,61,34,83,34
,32,116,121,112,101,61,34,102,108,111,97,116,34,47,62,10,32,32,32,32,32,32,32,32,32
,32,32,32,32,32,60,112,97,114,97,109,32,110,97,109,101,61,34,84,34,32,116,121,112,101
,61,34,102,108,111,97,116,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47
,97,99,99,101,115,115,111,114,62,10,32,32,32,32,32,32,32,32,32,32,60,47,116,101,99
,104,110,105,113,117,101,95,99,111,109,109,111,110,62,10,32,32,32,32,32,32,32,32,60,47
,115,111,117,114,99,101,62,10,32,32,32,32,32,32,32,32,60,118,101,114,116,105,99,101,115
,32,105,100,61,34,112,67,117,98,101,83,104,97,112,101,49,45,118,101,114,116,105,99,101,115
,34,62,10,32,32,32,32,32,32,32,32,32,32,60,105,110,112,117,116,32,115,101,109,97,110
,116,105,99,61,34,80,79,83,73,84,73,79,78,34,32,115,111,117,114,99,101,61,34,35,112
,67,117,98,101,83,104,97,112,101,49,45,112,111,115,105,116,105,111,110,115,34,47,62,10,32
,32,32,32,32,32,32,32,60,47,118,101,114,116,105,99,101,115,62,10,32,32,32,32,32,32
,32,32,60,112,111,108,121,108,105,115,116,32,109,97,116,101,114,105,97,108,61,34,108,97,109
,98,101,114,116,50,83,71,34,32,99,111,117,110,116,61,34,54,34,62,10,32,32,32,32,32
,32,32,32,32,32,60,105,110,112,117,116,32,115,101,109,97,110,116,105,99,61,34,86,69,82
,84,69,88,34,32,115,111,117,114,99,101,61,34,35,112,67,117,98,101,83,104,97,112,101,49
,45,118,101,114,116,105,99,101,115,34,32,111,102,102,115,101,116,61,34,48,34,47,62,10,32
,32,32,32,32,32,32,32,32,32,60,105,110,112,117,116,32,115,101,109,97,110,116,105,99,61
,34,78,79,82,77,65,76,34,32,115,111,117,114,99,101,61,34,35,112,67,117,98,101,83,104
,97,112,101,49,45,110,111,114,109,97,108,115,34,32,111,102,102,115,101,116,61,34,49,34,47
,62,10,32,32,32,32,32,32,32,32,32,32,60,105,110,112,117,116,32,115,101,109,97,110,116
,105,99,61,34,84,69,88,67,79,79,82,68,34,32,115,111,117,114,99,101,61,34,35,112,67
,117,98,101,83,104,97,112,101,49,45,109,97,112,49,34,32,111,102,102,115,101,116,61,34,50
,34,32,115,101,116,61,34,48,34,47,62,10,32,32,32,32,32,32,32,32,32,32,60,118,99
,111,117,110,116,62,52,32,52,32,52,32,52,32,52,32,52,60,47,118,99,111,117,110,116,62
,10,32,32,32,32,32,32,32,32,32,32,60,112,62,48,32,48,32,49,55,32,49,32,49,32
,49,32,51,32,50,32,50,48,32,50,32,51,32,50,50,32,50,32,52,32,50,32,51,32,53
,32,50,49,32,53,32,54,32,53,32,52,32,55,32,49,57,32,52,32,56,32,52,32,53,32
,57,32,49,56,32,55,32,49,48,32,55,32,54,32,49,49,32,49,54,32,54,32,49,50,32
,54,32,55,32,49,51,32,49,52,32,49,32,49,52,32,57,32,48,32,49,53,32,56,32,49
,32,49,54,32,49,53,32,55,32,49,55,32,49,48,32,53,32,49,56,32,49,49,32,51,32
,49,57,32,51,32,54,32,50,48,32,49,50,32,48,32,50,49,32,48,32,50,32,50,50,32
,50,51,32,52,32,50,51,32,49,51,60,47,112,62,10,32,32,32,32,32,32,32,32,60,47
,112,111,108,121,108,105,115,116,62,10,32,32,32,32,32,32,60,47,109,101,115,104,62,10,32
,32,32,32,60,47,103,101,111,109,101,116,114,121,62,10,32,32,60,47,108,105,98,114,97,114
,121,95,103,101,111,109,101,116,114,105,101,115,62,10,32,32,60,108,105,98,114,97,114,121,95
,118,105,115,117,97,108,95,115,99,101,110,101,115,62,10,32,32,32,32,60,118,105,115,117,97
,108,95,115,99,101,110,101,32,105,100,61,34,86,105,115,117,97,108,83,99,101,110,101,78,111
,100,101,34,32,110,97,109,101,61,34,117,110,116,105,116,108,101,100,34,62,10,32,32,32,32
,32,32,60,110,111,100,101,32,105,100,61,34,112,67,117,98,101,49,34,32,110,97,109,101,61
,34,112,67,117,98,101,49,34,32,116,121,112,101,61,34,78,79,68,69,34,62,10,32,32,32
,32,32,32,32,32,60,114,111,116,97,116,101,32,115,105,100,61,34,114,111,116,97,116,101,90
,34,62,48,32,48,32,49,32,48,60,47,114,111,116,97,116,101,62,10,32,32,32,32,32,32
,32,32,60,114,111,116,97,116,101,32,115,105,100,61,34,114,111,116,97,116,101,89,34,62,48
,32,49,32,48,32,48,60,47,114,111,116,97,116,101,62,10,32,32,32,32,32,32,32,32,60
,114,111,116,97,116,101,32,115,105,100,61,34,114,111,116,97,116,101,88,34,62,49,32,48,32
,48,32,48,60,47,114,111,116,97,116,101,62,10,32,32,32,32,32,32,32,32,60,105,110,115
,116,97,110,99,101,95,103,101,111,109,101,116,114,121,32,117,114,108,61,34,35,112,67,117,98
,101,83,104,97,112,101,49,34,62,10,32,32,32,32,32,32,32,32,32,32,60,98,105,110,100
,95,109,97,116,101,114,105,97,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,116
,101,99,104,110,105,113,117,101,95,99,111,109,109,111,110,62,10,32,32,32,32,32,32,32,32
,32,32,32,32,32,32,60,105,110,115,116,97,110,99,101,95,109,97,116,101,114,105,97,108,32
,115,121,109,98,111,108,61,34,108,97,109,98,101,114,116,50,83,71,34,32,116,97,114,103,101
,116,61,34,35,108,97,109,98,101,114,116,49,34,47,62,10,32,32,32,32,32,32,32,32,32
,32,32,32,60,47,116,101,99,104,110,105,113,117,101,95,99,111,109,109,111,110,62,10,32,32
,32,32,32,32,32,32,32,32,60,47,98,105,110,100,95,109,97,116,101,114,105,97,108,62,10
,32,32,32,32,32,32,32,32,60,47,105,110,115,116,97,110,99,101,95,103,101,111,109,101,116
,114,121,62,10,32,32,32,32,32,32,60,47,110,111,100,101,62,10,32,32,32,32,60,47,118
,105,115,117,97,108,95,115,99,101,110,101,62,10,32,32,60,47,108,105,98,114,97,114,121,95
,118,105,115,117,97,108,95,115,99,101,110,101,115,62,10,32,32,60,115,99,101,110,101,62,10
,32,32,32,32,60,105,110,115,116,97,110,99,101,95,118,105,115,117,97,108,95,115,99,101,110
,101,32,117,114,108,61,34,35,86,105,115,117,97,108,83,99,101,110,101,78,111,100,101,34,47
,62,10,32,32,60,47,115,99,101,110,101,62,10,60,47,67,79,76,76,65,68,65,62,10
};


const unsigned char UploadTest::diceTexture[] = {
137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,1,128,0,0,1,0,8
,6,0,0,0,186,119,161,103,0,0,0,1,115,82,71,66,0,174,206,28,233,0,0,0,6
,98,75,71,68,0,255,0,255,0,255,160,189,167,147,0,0,0,9,112,72,89,115,0,0,11
,19,0,0,11,19,1,0,154,156,24,0,0,0,7,116,73,77,69,7,220,2,6,18,51,57
,66,79,224,110,0,0,32,0,73,68,65,84,120,218,237,189,119,152,93,213,117,247,255,89,251
,156,219,166,55,9,21,4,136,42,154,4,238,6,23,12,216,6,227,216,201,147,184,16,151,159
,99,39,38,111,222,231,113,225,77,92,112,39,46,128,227,94,177,147,184,0,110,111,98,191,46
,96,154,49,166,136,18,219,113,4,152,46,144,144,80,25,77,159,185,115,203,57,123,253,254,56
,103,174,70,32,176,52,186,119,116,231,106,125,158,103,144,24,205,220,123,207,90,123,175,239,222
,107,151,37,27,151,44,81,12,195,48,140,3,14,103,38,48,12,195,48,1,48,12,195,48,76
,0,12,195,48,12,19,0,195,48,12,195,4,192,48,12,195,48,1,48,12,195,48,76,0,12
,195,48,12,19,0,195,48,12,195,4,192,48,12,195,48,1,48,12,195,48,76,0,12,195,48
,12,19,0,195,48,12,195,4,192,48,12,195,48,1,48,12,195,48,76,0,12,195,48,12,19
,0,195,48,12,195,4,192,48,12,195,48,1,48,12,195,48,76,0,12,195,48,12,19,0,195
,48,12,195,4,192,48,12,195,48,1,48,12,195,48,158,64,104,38,48,140,249,71,211,47,0
,73,191,158,10,255,52,255,246,167,126,215,48,255,219,12,192,48,12,195,176,25,128,97,236,239
,145,223,204,223,84,32,80,71,68,76,232,132,137,129,197,228,143,59,158,158,101,203,192,57,52
,142,25,223,178,149,137,187,254,135,238,225,17,132,42,104,6,196,163,106,227,126,243,191,9,128
,97,44,200,0,224,17,50,222,83,234,233,32,123,234,11,201,157,115,14,75,79,121,33,174,191
,7,31,134,56,192,227,233,141,35,162,145,81,70,111,186,133,201,171,126,9,183,221,74,102,120
,12,21,53,99,154,255,247,25,217,184,100,137,181,36,195,152,39,102,242,185,46,12,41,63,227
,89,44,251,244,197,132,71,28,65,32,14,37,34,144,0,84,80,85,68,4,80,34,245,136,4
,168,198,20,239,189,151,109,31,253,24,217,219,110,67,188,183,53,0,243,255,62,249,223,4,192
,48,230,145,72,20,159,201,210,241,190,247,177,232,109,111,67,50,57,16,77,58,177,10,136,162
,179,186,116,242,125,69,69,64,21,16,180,90,102,235,191,125,139,201,75,46,162,48,93,54,5
,48,255,207,153,224,221,29,29,31,53,183,24,198,60,5,128,92,150,158,207,126,134,158,115,223
,72,152,205,34,2,50,51,142,147,164,203,207,140,234,106,253,90,210,239,137,36,63,31,132,20
,214,156,140,95,113,8,197,155,111,36,172,70,102,88,243,191,9,128,97,52,39,138,199,17,132
,33,249,247,191,159,222,115,223,4,78,8,220,62,108,194,19,165,237,152,85,16,134,148,111,93
,139,243,201,0,18,108,66,96,254,223,115,108,27,168,97,52,24,47,130,243,158,233,103,60,147
,69,111,123,43,97,38,32,19,4,251,244,154,206,5,4,153,144,190,183,191,157,232,244,83,241
,78,65,4,229,233,247,141,27,230,127,19,0,195,152,71,2,47,148,123,123,88,118,201,69,72
,38,95,215,215,150,92,142,229,255,248,126,202,97,136,122,11,253,230,255,189,20,18,115,143,97
,52,106,226,159,124,85,157,39,251,252,83,201,28,121,116,221,243,51,162,158,224,184,227,200,156
,241,82,212,61,249,189,13,243,191,9,128,97,236,199,0,16,160,20,94,245,74,156,128,212,57
,44,171,8,42,1,93,231,189,5,209,204,147,222,219,68,192,252,111,2,96,24,251,145,137,129
,197,244,158,114,42,74,180,115,165,174,110,65,70,9,5,218,143,88,133,246,118,155,177,205,255
,123,133,9,128,97,52,152,182,227,79,128,254,94,2,13,27,112,130,87,16,113,4,93,157,4
,253,189,184,180,83,207,124,217,142,32,243,191,9,128,97,236,71,58,151,45,195,5,33,234,36
,221,229,93,215,238,159,116,246,108,142,108,161,205,140,109,254,183,25,128,97,52,21,206,33,62
,57,200,89,239,241,159,162,168,40,78,5,21,203,251,155,255,247,238,61,76,0,12,163,209,120
,143,119,224,26,16,153,157,202,204,1,82,188,170,9,128,249,223,4,192,48,154,137,241,45,91
,113,113,117,231,105,255,58,19,35,68,229,50,213,82,217,140,109,254,223,59,1,49,247,24,70
,61,167,228,201,73,204,217,71,114,38,215,173,35,26,25,37,110,64,14,64,197,131,247,68,19
,19,196,67,195,59,7,157,233,151,205,6,204,255,38,0,134,177,31,233,26,218,193,232,205,191
,65,144,122,239,2,68,17,98,96,242,225,187,9,134,71,204,216,230,127,155,1,24,198,254,228
,137,119,180,59,137,25,191,242,151,120,173,214,253,32,16,120,136,35,38,190,241,109,132,120,151
,155,36,173,86,128,249,223,182,129,26,198,126,232,252,179,59,158,250,16,214,174,165,116,255,253
,117,47,229,40,42,20,255,120,47,209,175,110,124,210,251,91,240,55,255,219,12,192,48,246,123
,84,80,178,35,19,108,253,232,199,208,106,125,23,106,181,28,177,227,203,95,160,80,46,99,85
,34,205,255,123,139,213,3,48,140,121,72,5,168,40,110,211,38,74,93,157,228,87,159,148,20
,246,144,185,143,191,42,229,136,72,148,177,47,126,9,255,221,203,106,91,1,13,243,255,222,96
,51,0,195,152,175,128,224,149,201,79,95,194,200,79,127,142,250,125,139,214,26,10,59,46,187
,130,145,75,191,140,168,13,253,205,255,115,163,101,103,0,250,20,138,252,84,255,46,127,226,117
,108,112,101,212,131,76,165,66,241,230,27,32,8,200,175,89,131,134,1,146,214,124,213,218,230
,65,65,81,36,205,23,107,173,45,198,73,75,44,87,24,249,226,151,24,255,220,167,201,77,78
,213,90,167,181,209,230,239,255,205,230,255,150,45,10,239,159,102,186,163,79,209,0,100,55,206
,87,155,42,25,117,8,70,79,108,71,145,8,241,25,47,98,217,255,121,31,153,227,143,3,23
,34,120,156,8,78,2,84,61,136,36,237,88,99,170,177,34,62,162,116,239,3,108,255,234,231
,200,252,226,218,39,141,252,172,141,54,103,255,111,102,255,155,0,152,0,24,251,65,0,4,136
,69,168,100,50,100,206,56,131,174,183,191,133,194,145,171,8,59,59,112,217,92,114,197,151,130
,175,148,169,140,79,48,189,254,126,198,46,253,87,162,27,126,77,190,50,141,18,60,101,251,54
,154,95,0,154,197,255,173,47,0,146,252,143,58,33,10,3,114,71,173,34,122,201,11,233,88
,115,50,237,203,150,33,34,76,15,110,103,252,238,117,4,55,173,165,116,215,58,130,114,145,140
,119,120,212,58,149,209,32,81,80,148,228,246,46,239,20,231,51,104,95,15,174,191,151,76,33
,143,136,128,42,213,82,137,234,208,48,193,240,40,16,39,77,58,41,255,106,44,224,254,223,44
,254,63,0,4,192,19,119,119,211,246,218,55,210,246,234,115,40,172,58,22,114,121,2,39,187
,40,180,247,138,70,21,166,31,93,79,245,186,223,48,250,173,75,201,108,217,138,83,235,105,70
,99,103,5,187,237,152,179,70,164,79,55,154,53,22,102,255,111,22,255,183,164,0,40,160,170
,120,231,8,79,59,141,222,119,189,155,252,51,158,1,206,225,208,180,76,155,212,114,108,14,136
,53,49,179,56,208,24,42,143,172,103,236,243,159,103,234,39,63,38,235,61,90,247,155,188,13
,11,82,38,0,7,114,255,111,6,255,47,120,1,216,125,62,207,83,9,133,194,159,191,142,37
,23,127,156,160,173,115,110,14,42,151,217,254,111,223,96,234,162,207,144,137,167,17,239,118,113
,136,157,182,52,12,235,255,11,153,150,220,6,26,139,144,253,139,215,112,208,191,92,130,20,218
,112,115,117,147,11,201,159,244,12,226,165,7,225,175,249,117,173,169,169,9,128,97,88,255,55
,1,104,66,196,19,190,248,116,150,126,249,11,184,66,129,192,51,231,21,19,77,61,220,113,252
,9,140,111,221,12,119,223,83,155,8,90,240,55,12,235,255,11,157,150,74,37,170,66,220,221
,77,239,249,231,19,182,117,36,121,62,39,251,100,156,208,5,184,48,228,160,11,46,64,87,29
,65,76,100,193,223,48,172,255,183,4,173,181,150,36,208,241,218,55,144,63,249,228,250,79,149
,122,123,201,188,238,92,2,50,214,211,12,195,250,191,9,64,179,81,205,132,228,94,245,10,52
,168,255,99,105,32,44,250,171,215,48,189,98,185,117,52,195,176,254,111,2,208,108,100,143,94
,69,97,213,241,132,13,120,237,48,22,92,79,31,61,175,120,133,245,52,195,176,254,111,2,208
,108,248,23,191,0,87,40,224,27,80,9,85,197,39,167,243,142,59,198,122,154,97,88,255,55
,1,104,54,218,79,58,25,39,138,107,192,201,6,113,14,156,144,63,104,169,245,52,195,176,254
,111,2,208,116,13,96,233,210,218,45,122,117,31,1,104,114,114,48,219,217,99,61,205,48,172
,255,155,0,52,221,195,184,12,34,1,141,216,164,37,40,168,93,8,97,24,214,255,77,0,154
,146,210,224,54,116,198,89,117,111,0,73,65,134,106,113,210,122,154,97,88,255,55,1,104,54
,70,239,94,135,122,37,82,95,247,215,142,81,80,161,184,125,187,245,52,195,176,254,111,2,208
,108,100,111,186,5,239,43,184,6,60,150,120,65,99,65,30,124,200,122,154,97,88,255,55,1
,104,54,138,119,221,197,244,195,235,81,105,192,54,0,39,104,169,200,224,117,191,180,158,102,24
,214,255,77,0,154,238,97,202,83,196,215,253,230,233,47,218,158,235,20,208,123,118,92,125,37
,237,247,222,143,39,121,11,181,62,103,24,214,255,77,0,154,131,208,11,195,223,190,148,242,35
,15,215,253,181,181,82,98,250,242,31,130,70,214,211,12,195,250,191,205,0,154,13,65,200,108
,217,202,216,231,190,136,47,85,240,222,239,219,169,64,85,34,175,104,228,25,254,202,215,201,220
,121,27,62,221,8,102,55,130,238,129,249,108,150,100,254,183,254,223,212,180,84,61,0,151,54
,130,210,3,247,18,117,228,41,172,121,38,4,130,155,227,193,144,72,65,35,101,226,246,27,25
,251,192,135,145,168,138,32,233,251,24,123,18,0,118,118,78,195,252,111,253,223,4,160,161,35
,128,180,33,168,50,189,246,191,136,151,29,68,251,113,199,33,46,152,219,235,197,158,177,219,127
,195,230,183,158,71,199,212,20,51,245,161,45,152,237,93,0,176,217,146,249,95,172,255,55,103
,204,108,213,162,240,136,226,52,192,159,251,26,150,92,240,65,130,190,94,112,14,241,10,196,73
,163,80,7,233,164,46,86,95,107,177,49,32,213,18,67,95,189,148,169,47,127,133,236,212,20
,234,236,12,112,163,210,2,86,224,220,252,111,253,223,4,160,97,13,206,175,58,138,236,185,231
,178,232,47,94,139,235,239,77,190,41,2,66,154,209,243,196,94,113,8,113,113,138,161,95,94
,73,233,138,31,17,222,121,91,109,67,129,5,169,250,224,159,98,228,102,226,106,254,183,254,111
,2,208,152,6,128,226,112,76,31,124,48,61,231,156,131,28,187,138,252,210,37,100,59,187,0
,136,138,147,76,15,14,162,15,60,204,224,117,87,211,254,199,123,113,26,19,35,184,116,222,103
,2,240,244,54,22,20,167,142,216,37,35,170,170,64,165,167,151,142,254,1,194,66,1,31,69
,148,70,71,169,238,216,78,182,90,197,105,210,249,84,64,212,4,192,252,111,253,223,4,160,73
,70,36,54,50,157,139,205,148,170,83,178,253,203,169,158,250,28,22,157,253,74,228,89,39,82
,232,29,32,200,181,225,227,136,202,196,56,213,7,31,98,242,250,95,81,188,225,122,120,224,62
,50,30,196,55,230,254,22,195,252,111,253,223,4,192,26,192,60,216,204,231,11,116,188,227,157
,116,156,251,215,100,7,122,81,129,128,0,156,18,167,35,48,81,136,137,64,3,124,121,154,233
,223,255,142,29,231,191,151,236,99,27,176,13,163,230,127,235,255,38,0,251,101,250,250,116,13
,192,120,234,78,163,226,8,124,5,127,194,201,44,250,250,151,40,28,113,84,205,114,162,130,186
,217,35,59,65,241,224,5,92,114,177,22,42,196,197,113,182,127,238,11,84,46,253,26,97,172
,38,3,230,127,235,255,243,72,75,109,3,157,147,2,62,205,151,241,244,157,38,208,42,242,178
,115,232,253,230,215,232,56,248,80,68,164,246,149,44,176,237,58,142,18,210,127,159,249,83,192
,101,115,100,79,61,133,106,223,98,162,91,111,70,226,216,12,108,254,183,254,111,2,96,52,115
,231,23,32,62,97,13,61,151,126,133,92,223,0,161,155,251,50,153,40,228,79,92,77,37,159
,37,190,229,150,134,148,244,51,204,255,198,110,108,127,160,167,128,140,189,239,173,30,33,202,228
,89,246,171,107,40,28,121,116,253,82,11,165,18,143,190,253,109,232,245,55,16,170,214,198,97
,182,3,203,252,111,52,6,179,173,177,119,13,198,59,34,167,116,190,243,157,228,15,63,178,190
,177,37,151,103,233,7,63,68,208,209,137,90,18,206,252,111,152,0,24,205,69,236,60,153,190
,101,116,188,225,141,245,79,148,138,146,89,185,146,224,229,103,227,68,109,45,198,252,111,254,55
,1,48,154,10,133,234,11,158,67,182,175,39,57,77,89,207,254,143,160,65,150,182,183,189,9
,50,237,22,0,204,255,230,127,19,0,163,153,136,156,176,248,172,87,66,144,148,201,171,111,108
,81,2,85,122,78,88,13,203,150,153,177,205,255,134,9,128,209,76,84,186,123,145,103,159,132
,243,1,190,206,173,71,0,113,224,196,17,31,190,162,86,121,201,48,255,27,38,0,70,19,208
,49,48,64,190,167,63,233,168,117,223,63,150,76,250,213,57,218,251,250,205,216,230,127,195,4
,192,104,38,194,124,129,32,155,39,106,212,13,94,170,40,66,144,203,155,177,205,255,134,9,128
,209,76,248,184,138,247,85,66,92,3,174,111,153,217,252,167,104,84,53,99,155,255,13,19,0
,163,153,40,141,142,81,157,24,7,15,42,245,142,0,130,34,136,42,211,227,227,102,108,243,191
,97,2,96,52,19,209,142,237,248,135,214,83,145,42,196,245,223,5,226,213,163,177,18,111,218
,98,198,54,255,27,38,0,70,51,145,171,86,25,253,213,181,233,85,191,245,126,117,65,69,152
,26,217,130,108,220,104,198,54,255,27,38,0,70,83,53,24,133,169,235,110,192,79,151,17,234
,125,115,163,130,135,226,143,126,78,52,57,108,7,129,204,255,230,127,19,0,163,185,2,128,160
,15,63,64,229,247,191,197,215,187,249,40,248,241,9,70,190,247,109,50,177,88,0,48,255,155
,255,77,0,140,102,34,114,144,175,84,121,252,61,239,65,167,38,234,155,0,80,101,232,199,223
,39,255,232,6,107,152,230,127,99,30,176,122,0,198,222,117,82,192,139,144,27,25,101,50,138
,200,159,122,10,170,224,246,225,62,248,88,99,34,47,20,239,184,157,137,243,255,137,160,26,49
,83,106,220,48,255,27,38,0,70,179,77,27,5,226,63,252,158,74,255,98,10,199,175,38,8
,231,30,0,34,15,147,235,214,49,242,158,127,36,220,186,117,86,37,41,195,252,111,152,0,24
,205,217,120,188,18,223,114,11,229,66,134,246,213,107,32,8,17,73,70,110,170,154,94,22,185
,107,71,214,153,145,157,106,109,218,95,188,253,246,164,243,223,247,160,245,123,243,191,49,159,51
,58,171,8,102,236,45,126,86,58,64,0,117,142,232,244,151,176,236,35,31,36,60,236,112,112
,33,162,30,231,4,36,64,189,79,130,129,10,177,143,80,9,80,31,195,228,36,195,63,249,1
,165,79,124,134,160,56,181,219,215,54,204,255,134,9,128,209,196,1,64,128,178,19,130,246,78
,228,172,151,211,249,55,111,162,123,245,106,132,16,117,201,1,127,65,147,241,159,135,233,225,109
,20,127,244,83,134,47,255,46,249,13,27,9,116,103,253,39,11,0,230,127,243,191,9,128,209
,196,204,46,12,62,251,187,201,182,192,8,151,237,68,151,47,199,175,92,65,123,95,31,65,46
,143,70,17,197,241,113,116,243,22,216,176,145,104,98,7,161,79,246,149,235,172,84,193,238,95
,219,48,255,27,38,0,134,97,24,70,221,176,237,182,134,97,24,38,0,134,97,24,134,9,128
,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24
,134,97,2,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97
,2,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96
,24,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134
,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,2
,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24
,134,97,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97
,152,0,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0
,24,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134
,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24
,38,0,134,97,24,7,60,225,190,252,178,166,95,179,145,244,203,216,115,251,253,41,155,237,206
,206,205,96,111,243,127,125,108,247,167,108,230,159,230,223,204,255,11,243,89,154,197,255,54,3
,104,162,6,247,116,63,107,24,134,209,84,51,128,68,123,20,196,39,127,224,80,81,32,96,162
,189,131,104,229,10,122,15,63,2,233,232,128,74,149,233,199,31,167,244,192,195,116,14,15,19
,198,101,98,81,66,85,148,0,5,156,74,250,251,173,28,248,19,155,41,30,117,130,168,160,62
,198,7,121,198,22,13,208,126,236,49,228,150,45,3,9,16,31,51,189,237,113,198,239,186,135
,238,225,17,66,141,64,21,7,168,130,138,75,237,174,77,240,92,158,80,3,162,192,19,170,80
,42,228,40,29,126,20,253,135,175,36,219,217,69,92,169,48,177,249,113,42,15,220,75,219,200
,48,46,86,20,169,89,131,244,239,7,130,240,207,252,77,5,2,117,68,196,132,78,152,232,95
,76,219,113,199,211,185,124,25,56,7,113,204,248,214,109,76,172,251,3,221,195,35,8,85,208
,12,136,71,85,154,236,185,154,211,255,94,60,25,239,136,28,136,122,226,48,195,228,146,37,180
,31,117,52,157,7,29,4,206,49,189,99,7,227,15,63,68,126,211,102,50,149,34,226,93,50
,54,22,221,101,148,222,112,255,15,44,38,127,220,241,244,44,75,252,175,243,224,127,217,184,100
,137,238,219,195,196,196,146,232,72,57,151,39,56,233,25,100,206,120,1,139,206,60,147,182,21
,43,209,124,22,17,135,71,144,40,34,30,220,202,208,111,239,96,252,154,27,169,174,93,75,97
,199,86,130,200,129,139,16,117,45,31,4,20,65,81,68,29,177,40,83,125,221,100,94,112,42
,157,47,63,131,197,207,123,33,110,224,32,36,19,162,170,120,32,136,35,170,67,219,217,113,235
,90,198,175,186,30,189,243,118,114,67,67,4,154,204,33,220,126,14,102,179,27,116,53,155,161
,176,250,217,248,179,94,194,226,211,207,34,92,185,28,201,100,16,113,201,115,251,152,120,199,16
,227,119,220,201,228,213,87,225,111,185,133,112,104,7,168,236,81,42,172,21,240,179,254,204,168
,167,212,211,77,246,212,23,146,63,231,28,250,78,121,33,50,208,131,4,33,162,16,139,199,197
,17,209,200,40,163,55,221,194,228,85,191,132,219,110,37,51,60,134,79,7,74,205,147,2,106
,78,255,43,66,36,49,153,101,43,137,207,120,49,125,231,252,25,109,39,158,128,235,108,67,36
,4,39,120,95,69,139,211,148,239,187,159,237,215,92,143,220,112,3,242,192,189,248,216,215,250
,151,155,39,255,187,254,30,124,24,226,0,79,227,253,191,79,2,224,113,168,196,168,4,76,29
,126,24,203,222,247,94,186,94,116,38,65,62,135,23,69,92,50,43,136,101,103,160,242,170,4
,94,192,199,84,31,219,192,131,31,251,4,225,205,191,166,80,154,38,114,1,25,223,234,2,224
,81,81,202,185,54,74,207,56,153,163,63,246,9,50,71,31,137,132,142,24,33,64,146,145,135
,42,136,3,21,34,141,9,17,36,174,48,245,199,123,121,248,147,23,83,184,125,45,185,40,194
,169,236,183,25,64,45,0,136,224,15,57,140,190,143,124,152,252,105,167,19,228,66,80,79,32
,14,65,82,255,11,168,226,137,81,113,168,135,242,230,199,24,186,224,195,232,77,55,34,213,106
,58,34,58,48,4,192,133,33,229,103,60,139,101,159,190,152,240,136,35,8,196,161,68,4,4
,168,8,170,169,205,80,34,245,136,4,168,198,20,239,189,151,109,31,253,24,217,219,110,67,188
,111,14,1,104,98,255,107,91,27,238,77,111,166,255,127,253,3,225,192,0,1,30,47,66,136
,35,78,13,23,40,84,211,161,25,2,209,100,137,161,43,190,69,245,139,95,65,70,70,26,34
,0,79,233,127,9,64,5,175,138,147,198,251,127,78,2,48,227,120,7,76,100,133,240,236,87
,113,228,63,95,136,12,12,48,211,108,81,69,93,50,177,19,5,149,116,170,167,201,247,124,154
,12,113,149,50,131,255,239,199,12,93,120,17,217,209,173,100,124,208,226,1,64,152,108,203,211
,251,174,243,89,250,214,183,32,109,5,60,14,151,206,12,118,166,194,210,236,218,140,221,210,198
,163,40,65,177,196,230,239,124,139,137,127,249,28,237,211,83,243,30,0,102,26,177,138,36,233
,191,87,190,154,131,63,125,49,97,87,79,146,155,74,62,52,42,186,203,196,62,153,249,204,8
,92,50,126,81,31,49,248,147,31,51,249,225,15,145,29,25,107,249,57,64,36,138,207,100,233
,120,223,251,88,244,182,183,33,153,28,136,166,125,70,106,54,219,37,245,160,154,216,90,147,113
,178,86,203,108,253,183,111,49,121,201,69,20,166,203,243,110,178,102,242,255,236,89,200,236,32
,93,117,49,218,191,156,37,151,253,43,109,39,158,132,56,151,188,118,26,139,158,248,46,170,233
,103,77,251,155,0,211,143,172,103,243,59,222,65,230,183,191,75,126,71,101,159,3,238,159,242
,127,146,118,146,93,146,236,141,244,127,240,238,142,142,143,206,53,143,85,14,33,251,214,243,56
,236,195,31,198,245,246,2,46,241,189,128,136,226,82,107,139,36,33,95,210,230,32,146,124,207
,33,16,4,228,143,57,134,204,49,199,179,227,182,219,201,79,78,182,116,8,24,111,207,177,232
,146,75,88,252,250,191,70,218,10,8,14,100,198,46,137,221,124,250,63,78,146,78,180,211,158
,130,136,67,178,33,237,107,214,224,151,45,99,116,237,29,228,42,165,253,147,199,22,79,230,111
,255,158,37,255,252,207,100,58,59,211,239,73,173,229,62,49,171,59,211,193,32,109,19,2,226
,28,133,163,143,33,56,113,13,147,55,221,72,80,156,110,109,1,200,101,233,249,236,103,232,57
,247,141,132,217,108,98,131,153,176,50,203,102,187,4,26,153,221,111,64,130,144,194,154,147,241
,43,14,161,120,243,141,132,213,200,252,63,203,94,138,39,62,252,104,150,253,224,123,20,86,157
,176,51,248,215,62,215,110,126,87,100,214,143,36,214,15,186,123,41,188,244,12,198,31,124,132
,96,253,67,181,215,144,6,250,127,214,255,205,139,255,247,66,0,102,242,116,201,120,212,59,71
,124,246,171,56,236,99,31,70,219,58,144,32,13,232,50,147,55,115,169,32,200,172,37,30,193
,75,50,190,17,37,81,59,17,34,28,185,67,15,38,183,244,32,38,110,184,129,76,20,161,104
,58,18,94,200,139,131,10,234,146,209,60,74,148,203,211,253,238,243,89,252,215,111,192,7,33
,226,102,172,146,142,154,210,255,219,37,8,36,209,191,214,48,37,93,152,82,2,186,142,62,130
,82,24,82,190,237,118,194,216,227,165,241,131,193,100,38,7,1,16,188,226,207,88,244,201,143
,35,249,28,129,155,251,36,217,163,132,203,15,37,179,98,5,211,215,94,75,24,249,36,133,216
,50,67,1,197,227,8,194,144,252,251,223,79,239,185,111,2,39,251,100,51,68,105,59,102,21
,132,33,229,91,215,226,60,168,176,207,1,106,225,251,95,241,203,87,176,248,75,95,38,123,236
,113,73,172,17,153,107,239,69,242,109,180,159,246,2,38,175,251,53,110,120,104,151,79,36,45
,224,255,189,156,1,36,35,121,143,99,100,209,34,86,125,253,27,132,139,23,19,56,87,83,119
,169,5,177,157,31,228,201,70,147,90,192,19,132,192,9,65,16,208,118,228,17,76,12,110,99
,250,158,63,146,241,233,238,34,22,238,194,112,50,153,75,70,248,145,8,83,207,126,54,71,124
,252,159,145,124,97,150,205,118,6,248,221,141,102,102,6,6,187,140,158,16,156,19,8,51,116
,157,120,60,143,223,189,142,112,195,163,4,58,43,223,218,32,102,22,167,42,61,253,44,191,236
,223,201,245,246,237,91,67,6,156,56,130,192,145,59,242,72,166,6,183,83,186,103,29,97,11
,173,5,121,17,156,87,74,207,126,14,7,127,234,19,4,217,236,62,219,76,196,225,2,71,254
,164,147,24,186,235,119,184,71,55,212,6,91,218,64,17,104,102,255,171,42,113,6,58,222,245
,79,244,190,250,207,8,156,204,57,248,207,204,10,2,231,200,180,181,35,171,142,102,226,170,95
,64,165,178,139,48,201,2,247,191,219,243,96,230,211,145,187,80,14,179,172,120,255,7,9,15
,94,90,223,128,153,201,178,236,127,255,111,162,190,197,160,65,146,10,89,208,187,224,5,239,34
,68,161,216,219,207,17,23,94,8,133,182,250,190,69,91,129,163,206,127,23,227,249,28,170,190
,182,187,168,145,147,154,56,8,88,116,225,133,100,23,215,215,255,18,132,244,191,235,29,4,221
,3,248,22,74,4,6,94,40,247,246,176,236,146,139,144,76,190,190,54,203,229,88,254,143,239
,167,28,134,168,159,7,213,108,98,255,139,115,100,158,255,98,250,222,242,230,186,63,118,219,115
,158,75,225,213,127,78,102,14,219,47,155,217,255,110,111,130,25,233,226,100,245,176,131,233,125
,233,153,201,62,244,58,7,204,112,201,82,10,127,249,42,170,161,34,62,92,208,2,224,212,225
,52,32,150,136,240,212,231,146,59,102,85,221,199,102,170,1,217,99,78,32,56,253,52,98,23
,212,68,90,27,100,57,239,148,112,249,97,228,94,254,178,186,191,190,138,146,93,188,152,220,235
,94,139,136,111,216,51,204,99,2,16,5,170,206,147,125,254,169,100,142,60,186,238,67,115,81
,79,112,220,113,100,206,120,41,234,158,252,222,173,238,255,217,249,242,88,132,174,55,191,17,159
,201,214,191,47,135,208,243,255,189,153,114,71,219,147,115,244,11,216,255,123,21,193,189,36,75
,185,237,47,122,33,174,179,179,49,243,76,113,44,123,221,95,49,221,209,133,136,39,22,183,96
,131,128,23,37,198,17,7,33,93,103,191,34,73,219,212,125,116,174,84,242,33,7,191,241,205
,20,51,25,32,102,102,189,166,33,65,64,65,206,60,141,108,91,97,159,166,215,79,149,96,84
,2,218,223,244,90,180,208,221,50,2,16,160,20,94,245,74,156,212,255,208,158,138,160,18,208
,117,222,91,16,205,60,233,189,15,4,255,215,130,241,162,1,194,103,63,15,23,212,127,38,36
,26,146,59,250,88,178,167,189,120,143,119,2,45,4,255,187,61,119,142,18,120,40,102,51,244
,158,126,38,62,8,26,212,59,29,249,67,86,226,87,172,196,75,149,80,23,238,198,64,47,16
,168,103,114,160,159,197,207,59,5,47,113,3,146,76,74,70,133,194,81,199,66,119,247,206,45
,101,13,162,18,6,244,158,249,50,188,147,167,191,164,100,78,177,37,217,139,221,113,240,97,232
,138,131,91,38,5,52,49,176,152,222,83,78,69,137,118,174,212,213,209,102,161,64,251,17,171
,208,222,238,134,63,75,51,251,63,120,254,243,200,245,247,33,190,1,61,64,99,36,12,200,63
,239,185,45,229,255,189,88,3,8,240,46,38,238,237,167,112,220,241,233,66,99,3,12,45,10
,153,44,237,203,23,19,248,144,88,22,238,106,160,164,211,218,246,99,79,196,13,244,34,234,234
,222,0,16,65,156,35,219,221,77,190,183,23,209,128,153,19,194,174,1,30,242,189,3,4,39
,28,75,160,14,239,234,63,2,112,8,56,135,59,100,121,203,8,64,219,241,39,64,127,47,129
,134,13,184,234,36,217,26,28,116,117,18,244,247,214,252,126,32,250,95,78,57,21,231,92,67
,230,141,146,246,51,183,226,144,150,242,255,94,164,128,146,173,155,133,190,126,178,61,221,72,3
,182,28,38,123,93,29,56,71,182,167,7,47,14,167,11,249,190,186,100,15,127,110,217,18,8
,51,224,130,250,199,255,153,61,66,185,44,20,114,52,250,78,157,246,254,62,114,93,221,136,19
,156,214,63,5,128,115,136,4,228,123,251,90,70,0,58,151,45,195,5,97,114,247,19,245,183
,153,3,92,54,71,182,222,27,12,22,152,255,251,14,63,60,57,47,35,13,136,25,233,107,182
,245,14,180,148,255,247,34,5,148,6,252,108,22,159,205,52,44,55,171,201,17,65,52,147,105
,232,118,182,249,147,0,80,231,240,170,120,109,132,189,146,73,147,146,172,209,52,58,103,30,100
,179,184,76,14,223,160,119,82,31,35,64,144,201,180,206,221,64,206,37,247,37,106,253,199,166
,138,162,162,181,211,227,141,94,55,105,70,255,207,60,115,216,214,145,254,93,26,208,207,60,32
,184,108,182,165,252,191,87,82,169,2,84,202,184,74,181,97,183,80,201,140,8,84,42,233,135
,91,192,203,128,154,76,194,196,43,210,160,145,185,204,242,195,76,248,111,164,197,226,74,153,40
,42,205,58,151,80,239,231,113,40,49,90,169,180,142,0,120,143,119,208,128,140,73,122,23,84
,98,40,175,218,112,1,104,54,255,207,94,236,172,22,39,210,131,166,141,136,75,201,17,88,63
,151,83,247,77,236,255,189,88,3,72,12,80,30,25,166,58,50,130,106,242,134,13,25,51,123
,79,52,54,134,168,103,161,239,3,17,160,180,101,27,18,71,136,106,221,55,1,205,220,19,164
,229,10,90,42,211,232,49,224,244,240,8,213,145,81,196,211,152,81,96,250,241,39,199,70,91
,38,5,52,190,101,43,46,174,210,160,152,73,140,16,149,203,84,75,229,134,63,75,51,251,127
,124,253,35,201,231,82,223,160,222,172,76,141,14,183,148,255,247,50,5,20,32,67,219,40,221
,123,31,234,21,37,174,187,129,99,175,80,169,50,181,121,51,94,26,51,157,107,116,202,199,51
,179,65,66,80,34,38,238,186,15,29,26,194,163,13,217,150,23,121,165,58,54,74,101,120,188
,118,93,199,204,103,168,251,148,115,120,16,253,227,125,84,164,138,139,235,191,163,193,251,24,31
,43,238,209,77,179,236,184,80,253,159,48,185,110,29,209,200,40,113,3,114,0,42,62,25,48
,77,76,16,15,237,12,78,7,162,255,227,181,183,162,62,162,17,227,82,175,49,18,11,178,97
,115,75,249,127,143,5,192,249,100,244,218,86,245,108,187,238,106,28,138,147,122,223,220,41,224
,2,74,27,30,129,77,27,137,92,146,215,94,168,36,87,219,134,116,13,111,99,112,237,77,59
,111,192,168,115,3,80,98,138,15,222,131,142,13,165,215,111,55,238,70,213,92,172,236,184,250
,42,2,13,240,65,253,119,52,248,32,100,252,177,245,196,155,30,107,153,25,64,215,208,14,70
,111,254,77,146,70,168,183,255,17,98,96,242,225,187,9,134,71,26,254,44,205,236,255,104,237
,173,76,15,13,161,141,104,254,18,18,251,136,209,181,183,180,148,255,247,88,0,98,151,46,54
,32,20,111,89,139,142,141,213,127,191,173,42,82,173,242,248,247,255,47,249,137,113,80,71,176
,192,134,128,187,156,76,116,137,104,102,169,48,122,213,213,184,40,174,251,8,64,212,17,150,171
,108,249,238,21,116,84,42,128,224,148,61,62,173,184,247,239,167,84,126,115,51,58,81,196,213
,121,168,165,40,18,69,20,191,253,61,180,52,190,96,3,254,19,237,238,36,102,252,202,95,226
,181,218,128,218,13,30,226,136,137,111,124,27,33,222,197,239,7,154,255,227,29,67,68,119,220
,137,171,255,113,27,188,70,148,239,185,27,189,249,166,150,242,255,94,111,3,85,148,220,134,205
,236,184,246,151,117,79,1,33,74,117,219,99,76,93,249,115,178,177,39,244,11,179,68,100,205
,248,154,92,137,173,222,193,109,191,165,120,223,61,13,72,2,41,147,247,222,67,245,198,91,112
,170,201,58,3,141,43,20,34,42,200,166,141,140,93,243,243,186,167,231,68,133,202,224,86,166
,126,246,179,164,104,208,2,14,254,179,63,189,250,16,214,174,165,116,255,253,117,47,229,40,42
,20,255,120,47,209,175,110,124,210,251,31,104,254,207,120,207,228,21,151,83,45,21,27,160,0
,176,227,242,203,201,22,75,45,229,255,189,16,0,135,226,112,64,62,170,176,233,211,151,16,61
,190,165,190,15,83,137,216,242,175,223,161,125,235,214,100,55,128,72,154,226,88,184,99,193,100
,123,166,35,63,50,204,195,159,188,8,234,125,223,253,244,52,91,190,250,117,122,139,19,232,172
,247,108,20,222,65,46,82,70,47,252,56,149,29,91,235,251,226,113,196,208,55,190,70,110,219
,54,28,45,84,26,76,148,236,200,4,91,63,250,49,180,90,223,133,90,45,71,236,248,242,23
,40,148,203,204,199,88,169,153,253,31,16,80,90,123,43,147,223,251,81,221,159,123,250,15,191
,67,126,246,51,252,92,6,189,77,236,255,61,190,14,90,210,7,153,209,149,252,212,20,35,27
,55,211,249,178,211,240,100,112,129,123,114,232,73,213,110,102,20,47,233,94,213,217,63,23,161
,84,99,133,106,133,225,43,127,201,212,197,23,147,143,162,244,93,210,34,40,11,185,239,167,101
,253,4,69,54,111,102,186,179,139,194,154,99,129,16,231,116,86,65,236,89,187,119,102,85,6
,210,93,164,100,103,9,237,74,197,131,175,178,233,171,95,199,93,118,25,65,173,162,81,99,239
,209,23,210,43,46,166,167,41,174,223,64,254,172,151,33,42,184,96,238,135,111,170,113,76,20
,193,228,149,63,167,252,241,79,225,210,61,215,11,125,27,232,236,207,175,162,184,77,155,40,117
,117,146,95,125,82,122,11,248,220,109,86,41,71,68,162,140,125,241,75,248,239,94,86,219,10
,56,31,207,212,172,254,87,32,240,202,212,93,119,145,63,245,185,72,223,34,212,145,150,86,156
,67,112,245,158,40,82,226,209,33,182,189,233,45,4,219,182,179,55,7,45,23,130,255,231,92
,17,76,0,125,100,61,67,99,211,244,159,242,28,36,155,165,86,204,68,147,36,244,76,101,171
,180,4,204,206,130,5,53,29,73,140,233,162,152,193,235,175,102,219,135,63,66,199,248,216,110
,235,8,44,212,0,48,155,208,123,38,127,127,39,186,252,16,58,143,57,14,113,193,236,66,69
,128,75,46,221,158,93,25,72,117,151,218,0,137,100,56,92,28,179,237,242,43,24,255,252,23
,40,148,231,183,138,214,140,40,233,35,15,50,57,90,162,240,226,23,16,134,225,220,7,126,10
,35,191,186,154,177,247,189,151,112,170,72,171,34,10,211,255,117,7,186,98,37,133,85,199,38
,151,3,206,145,72,148,29,151,93,193,196,231,46,33,83,169,154,255,103,231,42,138,147,140,220
,124,7,157,47,127,57,174,187,123,238,2,160,80,29,25,226,145,191,123,59,185,187,239,222,231
,130,59,205,232,255,57,9,192,140,17,28,158,234,61,235,24,223,176,137,238,103,60,11,215,158
,79,82,54,226,192,179,179,26,79,90,184,92,4,240,105,69,177,153,12,98,185,202,246,239,95
,206,224,69,159,202,129,174,213,0,0,29,91,73,68,65,84,164,107,251,142,214,154,250,239,134
,108,57,98,252,214,155,136,68,104,95,189,26,9,221,108,73,197,145,140,126,16,65,102,29,236
,240,51,45,18,65,74,147,108,254,202,87,25,249,210,231,233,26,31,223,47,98,57,83,170,193
,175,251,45,165,71,55,80,120,206,115,113,133,182,218,12,196,171,79,107,194,238,250,233,98,245
,181,10,199,10,104,20,51,118,249,229,76,94,248,49,50,67,35,117,123,14,253,19,130,172,127
,66,172,159,248,115,245,250,92,153,74,133,226,205,55,36,165,80,215,172,65,195,32,241,179,204
,108,222,157,153,239,165,245,115,103,7,220,244,166,87,202,21,70,190,248,37,198,63,247,105,114
,147,83,212,163,84,225,66,245,191,60,97,180,61,243,126,193,248,32,227,215,94,79,246,184,99
,200,28,124,72,242,97,103,138,147,203,108,175,206,250,188,233,247,98,98,68,29,229,187,238,98
,203,63,253,31,242,183,223,81,43,117,188,175,179,210,102,243,255,156,4,64,106,166,131,172,143
,240,247,63,200,230,27,110,160,176,108,49,153,69,7,33,65,22,47,174,86,124,89,213,35,120
,98,132,216,167,105,143,114,133,242,3,247,178,225,179,159,33,250,226,55,41,140,79,16,168,111
,245,154,224,136,64,174,82,161,120,251,109,108,189,231,127,232,58,116,37,65,239,0,94,130,164
,144,139,196,8,146,22,117,73,23,222,85,241,81,21,87,46,49,181,238,127,88,255,207,23,161
,151,95,70,174,60,69,224,93,237,8,184,204,99,231,175,21,169,84,193,223,119,31,195,215,94
,77,102,197,114,220,146,37,104,144,73,83,86,233,20,215,107,82,235,88,73,78,69,226,136,163
,10,213,135,30,98,203,167,63,73,249,139,95,37,172,115,45,232,167,19,0,221,3,129,120,226
,207,73,221,252,239,8,171,49,165,91,110,101,248,238,255,166,112,248,81,48,208,159,44,219,171
,79,7,86,110,231,44,26,80,141,169,196,49,62,170,50,117,247,31,121,236,194,15,16,127,231
,50,194,74,117,151,79,118,160,250,127,247,59,95,28,110,108,140,177,159,255,28,95,173,224,86
,30,74,208,222,141,226,81,141,211,27,116,72,135,86,146,20,133,247,73,74,218,239,24,97,251
,79,127,200,232,63,190,151,224,190,251,147,87,147,250,44,172,55,155,255,101,227,146,37,115,30
,114,123,28,66,140,74,50,110,45,7,1,165,213,171,89,246,250,215,211,249,162,23,145,237,238
,68,218,218,240,65,6,231,35,180,82,37,154,152,160,244,224,3,108,254,209,143,137,175,185,134
,142,137,49,28,62,189,203,194,33,45,56,3,152,29,72,156,186,164,186,154,75,78,240,141,23
,10,112,198,105,28,118,238,155,200,30,123,12,97,119,7,46,147,75,242,131,42,104,165,76,101
,116,148,210,3,15,176,241,242,43,144,27,126,77,119,113,50,173,207,163,144,222,254,9,123,121
,175,71,29,159,75,128,200,37,219,79,227,19,87,211,246,55,111,162,239,69,47,130,174,46,194
,124,91,82,77,67,99,226,74,25,95,44,50,181,254,1,166,190,253,61,166,175,190,154,194,212
,20,42,245,47,252,185,187,221,195,238,105,4,64,254,132,0,184,122,249,127,214,251,197,34,84
,50,25,50,103,156,65,215,219,223,66,225,200,85,132,157,29,184,108,46,9,80,10,190,82,166
,50,62,193,244,250,251,25,187,244,95,137,110,248,53,249,202,52,74,240,148,207,103,254,103,214
,88,90,168,6,30,186,250,200,188,238,53,244,189,254,181,100,22,45,193,181,181,225,178,121,192
,227,163,42,81,105,26,63,188,131,161,95,92,67,233,59,223,34,187,105,43,226,124,237,133,90
,213,255,251,36,0,233,146,80,210,8,210,233,74,44,74,37,204,80,105,111,39,215,219,67,208
,222,137,134,25,196,199,232,244,52,229,241,49,252,216,40,133,106,149,108,122,59,154,74,82,202
,240,64,195,167,246,139,156,48,29,102,144,158,94,114,61,61,80,40,32,36,98,168,165,34,165
,209,17,116,116,148,66,53,34,84,143,234,76,221,95,135,204,164,140,246,179,192,165,147,214,100
,88,160,33,113,91,27,126,81,31,249,246,118,92,38,131,198,158,202,244,52,126,100,12,25,29
,67,181,146,46,240,107,109,34,222,16,1,72,62,18,234,132,40,12,200,29,181,138,232,37,47
,164,99,205,201,180,47,91,134,136,48,61,184,157,241,187,215,17,220,180,150,210,93,235,8,202
,69,50,222,225,211,107,181,27,99,179,212,94,154,84,217,114,62,131,246,245,224,250,123,201,20
,242,105,234,84,169,150,74,84,135,134,9,134,71,33,221,129,34,90,255,3,133,173,230,255,90
,59,16,77,252,47,16,170,195,135,89,162,254,110,194,174,46,50,249,2,78,32,42,87,168,78
,76,16,236,24,197,71,211,56,245,181,187,127,26,213,183,154,197,255,251,40,0,251,162,204,45
,159,237,217,227,81,193,211,79,179,159,126,52,187,240,68,111,126,252,191,83,0,60,113,119,55
,109,175,125,35,109,175,62,135,194,170,99,33,151,39,152,181,0,151,100,38,20,141,42,76,63
,186,158,234,117,191,97,244,91,151,146,217,178,181,238,87,30,155,255,231,183,255,239,110,244,93
,175,153,93,43,248,127,222,5,96,103,214,237,192,22,128,63,213,8,228,105,114,214,11,217,118
,179,211,6,210,200,247,80,197,59,71,120,218,105,244,190,235,221,228,159,241,140,164,208,72,58
,174,11,210,133,118,159,118,166,56,189,64,76,28,104,12,149,71,214,51,246,249,207,51,245,147
,31,147,245,158,70,108,174,245,7,160,0,204,135,255,255,212,251,53,131,0,52,139,255,247,139
,0,24,70,35,69,84,240,84,66,161,240,231,175,99,201,197,31,39,104,235,156,91,7,45,151
,217,254,111,223,96,234,162,207,144,137,167,17,239,108,22,107,180,20,206,76,96,180,26,145,56
,114,175,126,13,139,63,125,49,180,117,204,125,116,148,201,209,255,183,127,79,199,167,62,142,248
,140,25,214,48,1,48,140,166,70,60,153,23,191,132,165,151,124,10,151,207,237,211,101,130,234
,64,66,199,162,215,189,30,61,247,53,233,66,165,52,244,174,29,195,48,1,48,140,185,4,108
,133,184,187,155,222,243,207,39,108,235,72,242,252,251,112,218,210,1,161,11,112,97,200,65,23
,92,128,174,58,130,152,200,130,191,97,2,96,24,205,55,250,135,142,215,190,129,252,201,39,215
,253,165,131,222,94,50,175,59,151,0,75,5,25,38,0,134,209,116,84,51,33,185,87,189,2
,13,234,223,172,53,16,22,253,213,107,152,94,177,220,12,109,152,0,24,70,179,145,61,122,21
,133,85,199,19,54,224,181,195,88,112,61,125,244,188,226,21,102,104,195,4,192,48,154,13,255
,226,23,224,10,133,134,20,43,87,241,201,233,204,227,142,49,67,27,38,0,134,209,108,180,159
,116,50,78,20,215,128,147,45,226,28,56,33,127,208,82,51,180,97,2,96,24,77,39,0,75
,151,162,234,27,114,81,142,106,114,114,56,219,217,99,134,54,76,0,12,163,233,26,179,203,32
,18,208,136,77,154,130,130,234,2,175,79,103,24,38,0,70,139,82,26,220,86,187,153,178,254
,2,144,20,228,168,22,39,205,208,134,9,128,97,52,27,163,119,175,67,189,18,169,175,251,107
,199,36,213,164,138,219,183,155,161,13,19,0,195,104,54,178,55,221,130,247,149,164,162,82,189
,103,0,94,208,88,144,7,31,50,67,27,38,0,134,209,108,20,239,186,139,233,135,215,39,149
,210,234,222,83,4,45,21,25,188,238,151,102,104,195,4,192,48,154,174,49,151,167,136,175,251
,205,211,95,180,62,71,98,239,217,113,245,149,180,223,123,63,158,180,86,171,153,220,48,1,48
,140,230,32,244,194,240,183,47,165,252,200,195,117,127,109,173,148,152,190,252,135,160,145,25,218
,48,1,216,163,78,99,163,164,3,218,166,243,253,89,5,33,179,101,43,99,159,251,34,190,84
,193,123,191,111,167,130,85,137,188,162,145,103,248,43,95,39,115,231,109,248,253,124,29,180,249
,255,192,254,188,245,254,140,243,34,0,38,2,7,166,61,231,251,243,10,224,84,40,254,244,63
,25,252,247,175,227,171,49,170,115,127,247,72,65,35,101,236,182,27,153,250,234,87,147,242,145
,42,251,61,248,155,255,15,204,207,218,136,207,104,41,32,163,229,200,196,158,137,79,125,150,161
,255,248,33,18,199,115,126,157,192,123,38,238,184,145,77,111,61,143,92,177,104,134,53,90,142
,134,214,4,126,170,218,169,254,9,163,182,3,245,108,229,158,22,133,95,168,236,206,255,187,123
,102,215,0,187,34,138,211,0,127,238,107,88,114,193,7,9,250,122,193,57,196,43,16,35,46
,72,74,126,165,73,157,88,125,237,131,197,128,84,75,12,125,245,82,166,190,252,21,178,83,83
,168,219,127,69,225,91,189,205,187,121,126,191,221,217,118,79,127,174,213,252,63,239,2,224,15
,160,198,62,215,224,223,202,2,48,223,254,87,192,175,58,138,236,185,231,178,232,47,94,139,235
,239,77,190,41,2,50,83,224,209,19,123,197,33,196,197,41,134,126,121,37,165,43,126,68,120
,231,109,181,207,235,204,255,13,11,122,141,246,127,51,10,64,179,248,191,78,2,160,179,62,118
,210,169,138,65,64,169,183,155,182,129,1,36,151,71,162,136,234,216,24,213,193,33,58,42,101
,156,198,168,40,162,59,187,150,75,14,91,30,32,2,160,120,152,117,183,140,48,149,205,226,251
,122,200,247,245,35,153,12,82,169,80,30,25,65,118,12,211,22,85,147,145,107,146,156,168,189
,6,77,16,40,102,174,95,112,234,136,93,50,162,174,10,148,122,122,233,236,31,32,44,20,208
,40,162,52,58,74,117,199,118,178,213,42,78,147,103,87,1,209,6,11,0,138,195,49,125,240
,193,244,156,115,14,114,236,42,242,75,151,144,237,236,2,32,42,78,50,61,56,136,62,240,48
,131,215,93,77,251,31,239,197,105,76,140,224,210,6,89,119,1,16,146,26,150,234,240,206,35
,10,229,76,136,246,246,211,222,215,71,144,205,226,203,21,166,71,134,97,104,144,140,87,68,61
,78,93,50,113,209,102,106,203,187,247,127,165,167,151,142,212,255,62,138,40,141,142,81,221,177
,109,94,253,239,69,9,188,35,150,36,228,122,17,202,133,2,185,129,69,228,187,186,16,17,42
,147,147,20,7,183,147,41,22,9,125,226,11,144,70,220,41,216,116,254,159,147,0,248,93,166
,111,158,88,28,168,18,75,64,117,233,50,120,222,51,25,56,235,229,180,175,89,77,190,255,32
,200,229,209,56,38,30,31,103,250,161,7,24,186,241,6,38,111,188,153,240,222,251,200,251,42
,46,10,16,241,56,245,168,72,139,7,126,64,34,188,207,128,40,229,48,67,245,144,149,180,159
,246,124,122,206,60,157,194,170,227,200,246,46,194,5,33,113,84,33,26,25,166,248,199,187,217
,113,253,181,148,110,189,147,236,163,235,41,68,224,113,136,84,16,117,243,46,0,254,41,158,172
,234,148,108,255,114,170,167,62,135,69,103,191,18,121,214,137,20,122,7,8,114,109,248,56,162
,50,49,78,245,193,135,152,188,254,87,20,111,184,30,30,184,143,140,7,241,141,185,191,167,25
,167,225,126,214,159,130,39,10,2,194,229,43,9,206,120,33,221,47,61,139,224,248,99,200,119
,247,225,50,57,226,106,137,202,232,24,213,187,239,102,236,154,171,169,222,116,51,126,243,163,228
,34,135,175,149,168,199,252,255,116,159,81,4,175,17,46,223,73,229,196,227,232,61,231,207,200
,189,224,20,178,7,47,39,211,214,137,184,128,114,105,146,232,241,45,84,239,184,147,193,171,126
,129,251,253,31,200,142,143,225,85,106,159,203,181,168,255,247,81,0,36,41,148,161,142,209,238
,14,250,206,59,143,37,175,57,151,96,113,63,222,5,224,2,2,81,98,141,112,233,45,141,94
,149,32,246,232,116,137,226,239,126,199,253,31,250,32,125,143,172,39,208,42,146,6,197,86,198
,167,145,197,139,99,104,113,63,43,222,123,1,253,47,123,57,174,179,131,56,8,8,84,192,41
,49,74,136,195,43,160,30,231,61,241,248,4,67,215,92,195,230,75,46,166,127,251,22,2,15
,158,16,17,191,223,3,128,207,23,232,120,199,59,233,56,247,175,201,14,244,162,2,1,65,237
,89,28,14,81,136,137,64,3,124,121,154,233,223,255,142,29,231,191,151,236,99,27,246,219,144
,118,127,9,128,3,74,189,253,244,125,228,35,180,157,125,54,217,246,54,84,60,129,134,224,148
,8,37,68,64,193,227,241,8,209,248,20,197,171,127,193,232,71,63,65,102,108,40,29,121,55
,135,0,236,145,255,61,196,50,63,254,223,153,126,84,252,139,79,163,255,189,23,144,59,238,24
,194,32,196,59,33,100,102,86,144,248,34,82,197,161,104,172,84,55,111,101,235,103,63,5,255
,241,19,156,54,70,0,154,197,255,251,36,0,130,163,28,84,169,158,248,76,142,252,244,197,100
,142,61,22,117,97,242,161,60,201,194,153,104,205,183,94,64,53,201,181,122,64,212,227,71,199
,216,246,165,175,82,252,183,175,147,141,99,156,111,233,248,79,12,84,194,0,57,251,44,86,126
,224,131,184,21,135,36,150,20,65,53,73,77,39,87,25,36,247,79,206,216,73,82,49,16,245
,196,91,30,231,177,15,125,140,248,186,171,201,249,10,162,193,126,9,0,42,142,192,87,240,39
,156,204,162,175,127,137,194,17,71,213,90,134,168,160,110,246,200,78,146,167,241,178,51,215,167
,66,92,28,103,251,231,190,64,229,210,175,17,198,58,239,50,240,167,242,176,245,38,18,8,20
,130,115,94,201,192,199,63,70,246,160,165,73,42,192,37,2,169,242,196,13,19,30,231,29,234
,146,235,168,85,133,104,199,54,182,189,231,2,226,235,174,196,249,253,55,3,108,86,255,43,66
,76,68,16,182,211,241,249,139,233,126,229,171,112,217,92,45,61,173,104,82,221,109,214,111,36
,125,43,201,67,171,10,226,61,19,183,252,134,193,119,252,31,178,219,55,19,75,80,43,52,36
,45,228,255,224,221,29,29,31,157,107,167,137,37,38,58,235,149,172,252,220,231,201,29,113,228
,206,187,216,5,212,41,142,153,7,74,34,155,0,142,164,176,134,136,32,226,8,242,121,218,159
,247,60,202,61,7,49,122,231,173,228,43,173,125,210,178,18,56,178,127,123,30,135,125,228,195
,4,139,151,224,68,16,39,196,50,211,116,65,68,211,102,154,100,251,5,135,72,98,59,17,135
,235,234,162,251,204,51,24,153,152,162,124,207,93,100,99,221,47,65,51,208,42,242,178,115,232
,253,230,215,232,56,248,208,212,167,201,23,178,115,69,104,246,218,144,136,236,252,83,192,101,115
,100,79,61,133,106,223,98,162,91,111,222,167,109,155,115,26,1,61,205,87,163,194,103,230,237
,127,207,162,79,126,156,92,111,223,78,155,193,44,155,205,254,124,59,23,171,103,126,54,104,239
,32,127,214,75,153,30,43,225,215,253,174,182,78,97,254,223,153,150,214,254,197,116,252,251,55
,232,61,235,21,4,97,102,215,207,37,187,177,178,60,225,179,57,71,120,200,161,184,51,79,99
,234,150,59,9,134,7,211,222,185,175,109,163,185,252,191,151,2,32,181,133,71,21,97,106,245
,73,172,252,252,103,9,23,47,69,130,153,69,29,197,105,186,184,131,36,57,253,89,15,166,146
,104,176,160,181,108,79,53,128,174,19,78,32,10,67,202,183,223,78,232,253,172,123,221,23,246
,154,192,206,39,80,34,23,18,191,242,85,28,252,209,15,64,161,11,2,197,165,158,119,58,147
,82,147,89,243,43,65,113,137,96,234,78,219,121,241,120,23,210,247,252,231,49,246,232,38,228
,254,251,230,237,64,199,236,229,254,248,132,53,244,92,250,21,114,125,3,132,110,238,159,64,20
,242,39,174,166,146,207,18,223,114,75,67,74,58,54,5,34,196,175,254,115,22,93,248,17,92
,174,64,176,79,54,115,228,79,59,149,233,71,54,160,247,221,55,111,189,100,33,248,191,218,217
,73,219,39,63,73,215,153,47,39,216,109,192,223,211,135,85,50,221,3,4,167,63,151,210,79
,174,70,74,197,125,19,128,38,244,255,94,8,128,162,4,32,49,42,194,72,71,39,171,254,253
,187,228,143,56,130,32,112,187,40,213,174,127,178,203,225,249,93,70,5,146,124,133,226,112,97
,64,231,9,39,176,227,158,187,241,143,174,79,119,9,36,11,204,11,245,180,218,76,208,78,210
,95,202,196,17,71,178,234,155,255,74,166,167,151,32,72,2,124,173,57,73,109,103,34,233,56
,100,23,203,81,27,37,8,14,71,224,28,46,155,165,251,153,39,177,249,215,55,16,140,12,37
,54,211,68,132,27,85,185,74,68,81,17,162,108,158,165,255,247,251,116,172,56,116,159,58,63
,128,115,142,32,112,180,175,94,195,240,61,235,136,30,121,100,102,162,222,2,67,128,52,219,129
,39,58,116,37,135,92,246,29,114,29,93,251,212,249,1,92,224,8,131,144,182,231,62,135,225
,107,175,69,199,70,9,180,241,214,106,54,255,235,19,190,98,23,211,254,119,255,192,65,111,127
,59,161,147,185,7,127,18,225,8,2,161,208,187,136,234,146,1,74,215,95,15,113,204,174,137
,173,133,237,255,61,254,20,201,3,71,56,31,82,117,25,22,157,247,15,228,142,57,178,190,173
,171,173,192,161,239,255,39,166,123,123,16,117,196,228,112,11,248,34,9,47,201,161,35,231,3
,166,242,109,172,120,239,251,112,189,245,173,41,235,150,44,230,224,247,158,207,180,203,226,124,72
,236,4,105,160,100,58,239,136,156,210,249,206,119,146,63,188,190,254,151,92,158,165,31,252,16
,65,71,103,109,186,221,10,4,177,163,154,205,208,247,209,143,16,116,118,215,247,181,251,23,211
,255,161,11,146,122,5,243,48,84,106,110,255,11,114,236,137,244,253,175,243,234,254,220,61,127
,246,106,178,103,158,65,160,173,229,127,183,231,166,77,246,236,123,241,148,151,30,196,226,215,191
,14,117,117,94,124,20,71,118,229,49,100,79,127,25,222,9,66,188,160,47,18,18,146,17,121
,236,20,119,242,26,186,78,123,73,3,10,150,7,116,61,239,197,248,213,39,36,239,163,59,207
,7,52,130,216,121,50,125,203,232,120,195,27,235,63,216,20,37,179,114,37,193,203,207,198,137
,182,204,97,168,40,136,41,172,126,54,249,23,191,4,209,250,250,70,36,166,253,148,23,193,154
,19,32,189,250,174,145,93,166,153,253,239,5,186,223,242,183,208,213,83,247,231,14,50,142,142
,55,191,153,56,151,109,41,255,187,189,9,103,177,36,167,38,51,207,121,54,153,254,1,164,1
,201,90,205,4,44,123,227,27,40,22,10,56,42,120,89,216,215,21,37,159,223,83,56,253,116
,36,159,173,127,177,18,1,233,236,100,209,27,95,79,37,72,79,181,54,114,43,173,66,245,5
,207,33,219,215,83,119,49,19,4,13,178,180,189,237,77,144,105,111,25,1,112,10,241,89,167
,19,230,50,245,31,0,136,67,218,58,200,189,245,205,160,97,227,47,52,107,98,255,199,93,221
,132,47,120,46,18,214,191,213,120,2,218,159,253,92,244,89,207,108,41,255,239,149,0,132,234
,153,14,67,22,191,236,108,226,76,136,214,63,150,33,4,116,28,127,34,213,229,43,146,255,243
,11,57,4,56,2,141,152,108,239,96,241,75,206,76,211,51,117,238,52,233,214,181,238,231,157
,198,116,71,7,72,68,224,195,198,141,102,157,176,248,172,87,66,144,148,73,172,111,108,81,2
,85,122,78,88,13,203,150,181,76,10,168,220,150,103,201,233,103,1,190,254,209,89,133,128,136
,254,83,79,35,46,228,27,63,155,105,98,255,103,159,121,18,249,101,135,16,248,250,75,96,160
,138,203,229,232,122,201,75,90,202,255,110,111,222,72,197,83,233,238,33,127,242,73,56,17,26
,114,141,147,83,36,151,165,99,197,114,148,36,229,180,80,81,132,216,9,213,195,15,163,176,242
,48,26,182,99,59,8,200,244,245,80,232,233,75,206,4,136,111,88,213,170,74,119,47,242,236
,147,112,62,192,215,217,253,146,12,104,112,226,136,15,95,81,123,134,5,59,251,75,191,138,43
,143,36,60,98,5,78,131,6,92,117,226,17,23,66,87,7,44,238,195,165,118,60,80,252,63
,19,133,28,16,159,118,58,46,12,26,211,102,68,17,231,208,149,43,91,202,255,110,175,188,131
,80,24,88,76,190,191,175,97,115,115,1,188,19,178,125,61,233,14,154,133,57,3,208,153,255
,170,163,119,229,225,104,54,219,152,37,186,153,77,68,133,60,174,189,128,166,155,71,27,69,199
,192,0,249,158,254,164,163,106,35,30,70,80,231,104,239,235,111,153,25,64,255,225,71,32,65
,136,147,6,180,230,52,165,16,230,219,200,181,119,204,110,18,7,156,255,251,143,58,10,151,158
,151,169,127,63,75,122,111,251,192,226,150,242,255,94,196,36,69,84,144,124,1,159,207,38,167
,86,27,18,57,147,109,102,154,203,165,167,96,23,110,10,72,211,133,115,58,187,81,113,141,89
,156,213,196,102,4,1,62,155,73,222,65,93,195,14,52,133,249,2,65,54,79,212,168,27,188
,52,217,254,23,228,242,180,10,201,165,115,1,145,107,92,75,35,12,113,97,166,150,3,62,144
,252,63,243,204,97,79,79,114,71,78,3,238,19,211,244,152,190,155,67,187,108,102,255,239,213
,54,80,21,133,40,194,69,113,114,117,65,35,2,154,72,98,236,40,74,39,29,11,251,84,144
,138,34,149,114,178,250,223,128,134,41,51,179,36,239,17,239,147,41,102,3,119,208,248,184,138
,247,85,66,92,3,86,27,181,118,104,78,163,106,203,8,128,175,148,81,209,228,158,167,70,205
,155,53,198,251,248,73,1,160,213,253,191,203,57,128,114,169,113,231,95,82,17,208,168,210,82
,254,223,171,69,96,21,165,58,54,76,60,62,145,150,218,107,140,210,58,85,116,98,178,214,24
,22,34,51,199,184,188,120,74,143,111,65,162,136,186,175,154,167,179,12,84,209,74,5,63,93
,66,84,27,186,13,180,52,58,70,117,98,60,89,207,170,187,56,39,103,196,69,149,233,241,241
,150,17,128,137,45,91,208,56,2,245,117,247,205,204,173,194,81,165,76,101,122,186,225,207,210
,204,254,159,120,108,99,114,195,114,67,218,127,114,151,209,244,248,104,75,249,127,47,102,0,30
,241,1,126,199,54,166,31,126,132,216,123,148,184,238,193,76,21,168,68,76,108,217,150,164,79
,22,112,10,72,72,84,127,250,225,63,226,135,70,136,21,84,235,187,68,165,234,81,85,162,241
,49,42,99,227,233,8,72,27,182,8,24,237,216,142,127,104,61,21,169,146,94,96,84,87,255
,123,245,104,172,196,155,182,44,248,192,63,179,56,89,121,224,94,100,104,148,42,113,221,83,154
,30,5,31,163,19,83,232,200,216,172,239,31,24,254,159,189,8,92,190,243,118,188,122,124,3
,118,1,121,245,16,11,126,211,150,218,251,73,11,248,127,143,5,32,57,102,236,104,43,197,140
,220,120,45,65,146,169,107,72,10,40,26,220,138,110,218,64,236,60,126,1,95,12,35,154,108
,251,236,28,28,101,199,111,111,75,47,163,170,255,62,96,112,148,31,122,8,55,54,70,228,148
,70,238,156,205,85,171,140,254,234,218,244,170,223,6,72,166,8,83,35,91,144,141,27,91,102
,6,208,54,52,204,240,127,221,138,16,214,127,60,163,14,239,2,38,31,121,144,96,180,241,179
,166,166,246,255,77,183,83,157,28,67,27,146,107,119,168,40,83,191,255,93,75,249,127,143,77
,229,197,161,18,19,224,25,191,254,55,232,84,3,166,155,10,196,158,173,63,251,5,185,145,29
,36,231,0,22,102,167,79,238,37,73,52,56,140,99,70,175,185,22,87,173,255,8,0,5,169
,196,108,249,225,127,144,139,202,136,66,232,27,183,8,232,20,166,174,187,1,63,93,78,78,106
,215,251,97,60,20,127,244,115,162,201,225,214,57,8,230,99,38,175,188,14,98,223,128,29,90
,201,122,217,212,119,190,143,167,218,240,91,77,155,217,255,149,205,143,82,249,239,187,26,18,51
,20,79,188,113,3,229,235,175,107,41,255,239,133,86,198,233,219,9,193,67,15,82,252,239,223
,53,96,130,169,248,209,33,70,190,255,3,242,81,76,24,187,134,45,234,52,62,253,147,150,186
,75,77,20,221,122,39,213,141,143,52,192,102,158,210,195,247,81,190,246,26,178,113,68,198,187
,218,157,226,141,176,156,83,65,31,126,128,202,239,127,139,175,247,16,80,193,143,79,48,242,189
,111,147,137,165,133,78,2,59,162,181,55,19,109,126,172,1,119,28,41,165,245,143,50,125,245
,85,4,218,248,43,173,155,217,255,153,56,102,250,199,255,73,164,245,191,82,92,21,182,126,255
,7,228,134,71,90,202,255,123,225,65,135,243,73,1,207,108,52,205,3,31,254,16,58,50,90
,215,71,17,239,217,254,227,255,36,183,113,125,250,233,117,65,31,4,75,238,233,78,108,150,31
,220,202,67,159,248,4,84,42,245,181,89,165,204,166,127,255,38,237,147,99,32,1,73,93,163
,198,133,205,200,65,190,82,229,241,247,188,7,157,154,168,115,202,76,25,250,241,247,201,63,186
,129,133,125,1,200,19,36,218,9,217,193,65,182,126,248,67,16,215,119,119,147,68,17,131,223
,252,50,133,169,226,188,12,150,154,217,255,161,6,140,253,244,63,41,94,121,117,221,159,59,222
,180,1,255,163,31,204,105,35,71,51,251,127,143,175,131,22,92,109,56,235,84,200,142,142,82
,140,60,29,167,60,11,175,25,196,145,214,249,114,187,230,65,4,216,165,42,208,204,183,210,245
,112,81,188,7,141,171,76,222,126,7,59,222,247,1,58,75,165,218,48,90,22,242,24,80,168
,213,69,16,245,232,166,205,232,242,131,201,28,123,100,82,117,200,185,154,69,106,219,183,116,86
,17,157,154,25,101,23,155,169,8,113,164,120,95,101,199,247,126,72,229,203,95,163,224,227,25
,131,53,212,98,2,120,17,114,35,163,76,70,17,249,83,79,73,11,26,205,61,100,199,26,19
,121,161,120,199,237,76,156,255,79,4,213,104,118,227,89,240,36,39,51,29,178,113,35,149,67
,14,33,119,204,49,160,186,79,54,139,226,152,72,97,226,242,43,40,127,241,75,184,121,218,46
,221,204,254,247,34,100,34,101,226,129,251,41,188,244,12,164,173,51,217,18,61,199,117,55,85
,79,37,86,152,46,178,249,239,206,35,123,239,3,168,219,251,171,214,155,217,255,123,93,17,108
,230,174,250,64,161,120,215,58,202,189,253,180,29,127,60,65,38,168,5,127,209,164,210,179,79
,234,27,166,219,33,107,59,124,147,125,42,162,248,244,222,122,173,42,147,127,248,31,54,127,240
,2,58,54,111,158,101,182,214,232,252,154,254,45,140,99,70,239,188,147,236,177,39,144,95,113
,48,46,12,153,41,178,147,216,108,230,172,64,82,82,111,167,205,146,173,113,42,105,69,61,64
,171,85,70,174,185,134,193,79,126,130,246,137,137,121,181,151,75,31,44,254,195,239,169,244,47
,166,112,252,106,130,112,31,26,179,135,201,117,235,24,121,207,63,18,110,221,90,151,186,75,205
,132,206,154,225,78,223,182,150,240,196,53,100,86,28,74,16,236,75,0,80,198,175,189,150,209
,11,63,66,102,158,70,255,205,238,255,180,166,18,110,120,7,227,247,220,75,215,203,94,134,20
,10,233,181,53,115,152,185,41,248,169,34,143,125,226,99,100,174,186,122,206,3,210,102,246,255
,156,74,66,214,166,92,222,51,246,95,119,162,65,64,199,137,39,32,153,244,18,50,73,138,25
,207,148,57,155,41,102,34,170,181,128,152,236,41,18,136,149,137,219,111,102,211,135,62,64,219
,31,31,32,160,245,202,65,205,118,77,88,41,179,227,182,181,100,151,46,165,112,196,145,212,202
,249,202,172,131,53,181,42,106,250,4,155,37,213,214,168,86,24,190,242,39,60,254,145,143,208
,61,56,220,208,171,31,158,182,241,120,37,190,229,22,202,133,12,237,171,215,64,16,38,53,160
,73,14,243,137,60,185,35,235,204,200,46,157,74,139,42,197,219,111,79,58,255,125,15,182,82
,220,223,45,217,82,137,201,155,110,36,179,98,5,185,35,143,76,238,151,73,231,120,90,27,42
,60,141,205,68,160,26,49,241,139,159,51,242,190,247,144,27,30,221,111,207,210,172,254,119,8
,193,99,143,50,113,215,221,180,61,231,217,4,221,189,105,89,90,210,59,205,102,85,40,172,205
,175,119,90,95,213,35,8,241,182,237,108,253,151,139,112,223,253,30,226,235,115,242,185,217,252
,63,39,1,240,181,9,154,39,95,41,83,186,253,118,6,239,185,135,174,99,143,69,186,187,64
,29,170,130,119,73,201,67,175,73,153,102,85,136,188,39,70,33,138,209,225,17,182,253,224,50
,134,222,251,65,10,155,55,19,204,18,7,105,65,17,16,192,169,167,48,57,206,232,13,55,50
,185,125,144,174,227,142,134,182,54,212,11,126,166,164,38,201,254,94,193,163,10,213,216,227,85
,209,168,76,188,101,19,143,125,233,139,76,93,244,121,218,38,38,146,67,115,243,108,179,217,251
,139,37,142,137,111,185,149,225,187,239,162,109,245,113,208,213,141,71,64,125,58,153,113,168,79
,127,67,33,142,35,188,10,113,28,227,199,199,25,252,193,119,152,56,255,61,132,91,183,38,7
,154,97,175,170,45,45,36,223,207,140,108,131,226,52,83,215,92,205,212,224,118,242,39,158,128
,20,242,36,101,157,125,173,251,235,140,240,123,136,125,68,164,130,143,34,226,109,91,216,242,133
,207,50,253,241,79,18,20,167,119,57,9,123,160,251,127,215,5,208,36,229,50,116,213,85,4
,203,151,16,174,88,129,6,153,36,156,106,148,252,251,44,155,199,94,81,175,120,85,40,87,25
,253,195,29,12,190,227,157,200,181,215,37,115,112,105,77,255,203,198,37,75,116,46,13,128,90
,242,194,225,212,81,14,34,74,61,253,132,103,188,148,131,255,250,92,218,215,172,70,195,0,13
,93,250,51,113,146,7,171,70,248,173,91,121,252,23,191,100,252,138,203,201,109,124,132,182,72
,81,124,90,58,237,137,165,164,91,12,21,28,158,88,2,202,129,82,25,56,136,236,95,254,37
,135,188,246,175,200,30,186,18,205,8,72,144,174,166,120,188,10,174,26,81,126,120,61,155,127
,244,35,138,63,255,25,237,91,183,146,245,204,42,0,51,107,106,62,79,1,224,137,13,187,236
,132,160,189,19,57,235,229,116,254,205,155,232,94,189,58,217,247,236,168,141,111,64,81,15,211
,195,219,40,254,232,167,12,95,254,93,242,27,54,18,232,206,101,235,39,190,118,171,226,84,152
,10,149,92,247,0,225,235,95,71,199,155,95,151,22,86,119,32,46,13,5,30,209,36,21,49
,190,225,97,138,223,190,130,226,207,126,74,102,219,224,110,43,229,153,255,119,159,126,169,228,66
,130,163,86,209,126,222,223,210,125,246,89,100,114,237,224,4,21,183,115,110,173,138,143,74,140
,222,114,7,99,95,251,58,238,183,191,35,44,77,55,204,182,205,226,255,57,9,128,238,34,0
,66,205,197,170,68,1,148,218,58,137,150,47,167,99,249,50,178,125,3,104,46,143,68,17,241
,196,24,197,173,91,169,108,218,72,126,120,152,124,92,5,13,144,39,189,94,43,119,126,65,37
,78,103,192,73,189,227,82,32,148,58,123,144,21,203,105,95,182,148,76,119,63,154,201,32,213
,10,213,209,17,138,143,63,78,188,121,19,109,227,99,132,222,167,7,204,60,42,46,217,150,87
,155,58,206,115,78,251,9,223,77,182,5,70,184,108,39,186,124,57,126,229,10,218,251,250,8
,114,121,52,138,40,142,143,163,155,183,192,134,141,68,19,59,8,125,178,175,92,103,165,10,90
,109,244,255,212,54,76,250,76,236,4,167,30,41,116,227,15,89,1,135,44,163,163,167,15,201
,100,240,149,10,83,163,163,4,27,54,19,111,126,12,157,30,197,121,183,75,63,121,170,84,163
,249,191,150,120,71,189,67,157,7,50,184,129,197,68,43,15,38,119,208,65,228,59,186,65,148
,74,113,138,233,193,33,194,71,30,163,58,248,56,97,84,70,52,72,111,58,112,13,177,109,179
,248,127,78,2,96,24,134,97,180,192,76,212,76,96,24,134,97,2,96,24,134,97,152,0,24
,134,97,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97
,24,38,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24,38
,0,134,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24,38,0,134
,97,24,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24
,134,9,128,97,24,134,97,2,96,24,134,97,152,0,24,134,97,24,38,0,134,97,24,7,48
,255,63,68,172,77,155,81,245,169,233,0,0,0,0,73,69,78,68,174,66,96,130
};
